<?php
/**
* Récupère les informations de la base de donnée     
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-acl  
* @author    Pascal Martin 
* @copyright Boréal Informations Stratégiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

/**
* Récupère les informations de la base de donnée
* 
* @category   PHP 
* @package    Borealis-acl
* @subpackage DataStore  
* @author     Pascal Martin 
*/ 
class com_borealis_acl_datastore_PDODataStore implements com_borealis_acl_datastore_DataStore
{
    /**
    * @var com_borealis_acl_datastore_PDODataStore
    */
    private static $s_objInstance = null; 
    
    /**
    * @var PDO
    */
    private $m_objPDO = null;
    
    /**
    * Construit une nouvelle instance de com_borealis_acl_datastore_DatabaseDataStore
    * 
    * @return com_borealis_acl_datastore_DatabaseDataStore
    */
    private function __construct()
    {    
        $this->m_objPDO = new PDO(com_borealis_acl_SettingImpl::getPDODataStoreDSN());    
    }
    
    // @codeCoverageIgnoreStart
    /**
    * Destructeur, fermer la connection a la bd
    * 
    * @return void
    */
    public function __destruct()  
    {    
        $this->m_objPDO = null;
    }
    // @codeCoverageIgnoreEnd 
    
    /**
    * return instanceof com_borealis_acl_datastore_DatabaseDataStore
    * return instanceof com_borealis_acl_datastore_DataStore 
    * 
    * @return com_borealis_acl_datastore_DatabaseDataStore 
    */
    public static function getInstance()
    {
        if (self::$s_objInstance === null)
        {
            self::$s_objInstance = new com_borealis_acl_datastore_PDODataStore();
        }
        return self::$s_objInstance;
    } 
    
    
    
    /**
    * Delete in datastore
    * 
    * @param com_borealis_acl_object_Base $pobjBase Object
    * 
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return void
    */
    public function deleteByID(com_borealis_acl_object_Base $pobjBase)
    {
        $strQuery = self::getQueryByObject($pobjBase, 'PREPARE_DELETE_BY_ID');                  
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id', $pobjBase->getID(), PDO::PARAM_INT); 
        self::execute($objStatement);        
    }

    /**
    * Update name in datastore
    * 
    * @param com_borealis_acl_object_Base $pobjBase Object   
    * @param string                       $pstrName Nouveau nom de l'action
    * 
    * @throws com_borealis_foundation_exception_AssertException
    * 
    * @return void
    */
    public function update(com_borealis_acl_object_Base $pobjBase, $pstrName)
    {   
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName); 
        
        $strQuery = self::getQueryByObject($pobjBase, 'PREPARE_UPDATE'); 
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id', $pobjBase->getID(), PDO::PARAM_INT); 
        $objStatement->bindParam(':name', $pstrName, PDO::PARAM_STR);
        self::execute($objStatement);   
    } 
    
    /**
    * Insert in datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'action
    * 
    * @throws com_borealis_foundation_exception_AssertException 
    * 
    * @return int ID
    */
    public function insert($pstrType, $pstrName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);
        
        $strQuery = self::getQueryByType($pstrType, 'PREPARE_INSERT'); 
        $objStatement = $this->m_objPDO->prepare($strQuery);  
        $objStatement->bindParam(':name', $pstrName, PDO::PARAM_STR); 
        self::execute($objStatement);
        
        return self::selectByName($pstrType, $pstrName); 
    }
    
    /**
    * Return id from datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param string $pstrName Nom de l'action
    * 
    * @return int ID
    */
    public function selectByName($pstrType, $pstrName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrName);  
        
        $strQuery = self::getQueryByType($pstrType, 'PREPARE_SELECT_BY_NAME'); 
        $objStatement = $this->m_objPDO->prepare($strQuery);  
        $objStatement->bindParam(':name', $pstrName, PDO::PARAM_STR); 
        self::execute($objStatement);
        
        if ($objStatement->rowCount() === 0)
        {
            return null;   
        }
        else
        {
            $objArrayResult = $objStatement->fetch(PDO::FETCH_ASSOC);
            return (int)$objArrayResult['id'];       
        }
    }   
    
    /**
    * Return name from datastore
    * 
    * @param string $pstrType Type de l'objet
    * @param int    $piID     Identificateur de l'action 
    * 
    * @return string Name
    */
    public function selectByID($pstrType, $piID)
    {
        com_borealis_foundation_util_Assert::assertIsInteger($piID);
        
        $strQuery = self::getQueryByType($pstrType, 'PREPARE_SELECT_BY_ID'); 
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id', $piID, PDO::PARAM_INT); 
        self::execute($objStatement);
        
        if ($objStatement->rowCount() === 0)
        {
            return null;   
        }
        else
        {
            $objArrayResult = $objStatement->fetch(PDO::FETCH_ASSOC);
            return $objArrayResult['name'];       
        }   
    }
    
    
    /**
    * Return id and name from datastore of all object
    * 
    * @param string $pstrType Type de l'objet
    * 
    * @return ArrayObject
    */
    public function selectAll($pstrType)
    {    
        $strQuery = self::getQueryByType($pstrType, 'PREPARE_SELECT_ALL'); 
        $objStatement = $this->m_objPDO->prepare($strQuery); 
        return $this->getArrayIDName($objStatement);   
    }
   
    /**
    * Insert relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    public function insertRessourceAction(com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareInsertRessourceAction();
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id_ressource', $pobjRessource->getID(), PDO::PARAM_INT);
        $objStatement->bindParam(':id_action', $pobjAction->getID(), PDO::PARAM_INT);
        self::execute($objStatement);    
    }
    
    /**
    * Remove relation betwen Ressource and Action
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    public function deleteRessourceAction(com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareDeleteRessourceAction();
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id_ressource', $pobjRessource->getID(), PDO::PARAM_INT);
        $objStatement->bindParam(':id_action', $pobjAction->getID(), PDO::PARAM_INT);
        self::execute($objStatement);         
    }
    
    /**
    * Return id and name of all action linked to the ressource
    * 
    * @param com_borealis_acl_object_Ressource $pobjRessource Target ressource
    * 
    * @return ArrayObject
    */
    public function selectRessourceAction(com_borealis_acl_object_Ressource $pobjRessource)
    {
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRessourceAction();
        $objStatement = $this->m_objPDO->prepare($strQuery); 
        $objStatement->bindParam(':id_ressource', $pobjRessource->getID(), PDO::PARAM_INT);
        return $this->getArrayIDName($objStatement);  
    }
    
    /**
    * Insert member into role
    * 
    * @param com_borealis_acl_object_Role $pobjRole     Target role
    * @param string                       $pstrUsername Nom d'utilisateur
    * 
    * @return void
    */
    public function insertRoleMember(com_borealis_acl_object_Role $pobjRole, $pstrUsername)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareInsertRoleMember();
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id_role', $pobjRole->getID(), PDO::PARAM_INT);
        $objStatement->bindParam(':username', $pstrUsername, PDO::PARAM_STR);
        self::execute($objStatement);            
    }
    
    /**
    * Delete member into role
    * 
    * @param com_borealis_acl_object_Role $pobjRole     Target role
    * @param string                       $pstrUsername Nom d'utilisateur
    * 
    * @return void
    */
    public function deleteRoleMember(com_borealis_acl_object_Role $pobjRole, $pstrUsername)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareDeleteRoleMember();
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id_role', $pobjRole->getID(), PDO::PARAM_INT);
        $objStatement->bindParam(':username', $pstrUsername, PDO::PARAM_STR);
        self::execute($objStatement);    
    }
    
    /**
    * Return list of member register in this role
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role  
    * 
    * @return ArrayObject of Username string
    */
    public function selectRoleMember(com_borealis_acl_object_Role $pobjRole)
    {
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRoleMember();
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id_role', $pobjRole->getID(), PDO::PARAM_INT);
        self::execute($objStatement); 
        
        $objArrayResult = new ArrayObject();
        while ($objArrayRow = $objStatement->fetch(PDO::FETCH_ASSOC)) 
        {
            $objArrayResult->append($objArrayRow['username']);
        }        
        return $objArrayResult;   
    }
    
    /**
    * Insert permision for ressource action into role
    * 
    * @param com_borealis_acl_object_Role      $pobjRole      Role
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    public function insertRolePermission(com_borealis_acl_object_Role $pobjRole, com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareInsertRolePermission();
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id_role', $pobjRole->getID(), PDO::PARAM_INT);
        $objStatement->bindParam(':id_action', $pobjAction->getID(), PDO::PARAM_INT);
        $objStatement->bindParam(':id_ressource', $pobjRessource->getID(), PDO::PARAM_INT);  
        self::execute($objStatement);               
    }
    
    /**
    * Delete permision for ressource action into role
    * 
    * @param com_borealis_acl_object_Role      $pobjRole      Role
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return void
    */
    public function deleteRolePermission(com_borealis_acl_object_Role $pobjRole, com_borealis_acl_object_Ressource $pobjRessource, com_borealis_acl_object_Action $pobjAction)
    {
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareDeleteRolePermission();
        $objStatement = $this->m_objPDO->prepare($strQuery);
        $objStatement->bindParam(':id_role', $pobjRole->getID(), PDO::PARAM_INT);
        $objStatement->bindParam(':id_action', $pobjAction->getID(), PDO::PARAM_INT);
        $objStatement->bindParam(':id_ressource', $pobjRessource->getID(), PDO::PARAM_INT);  
        self::execute($objStatement);       
    }
    
    /**
    * Return id and name of all action linked to the role (With duplicate value)
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role
    * 
    * @return ArrayObject
    */
    public function selectRolePermissionAction(com_borealis_acl_object_Role $pobjRole)
    {
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRolePermissionAction();
        $objStatement = $this->m_objPDO->prepare($strQuery); 
        $objStatement->bindParam(':id_role', $pobjRole->getID(), PDO::PARAM_INT);
        return $this->getArrayIDName($objStatement);    
    }
    
    /**
    * Return id and name of all ressource linked to the role (With duplicate value)
    * 
    * @param com_borealis_acl_object_Role $pobjRole Target role
    * 
    * @return ArrayObject
    */
    public function selectRolePermissionRessource(com_borealis_acl_object_Role $pobjRole)
    {
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRolePermissionRessource();
        $objStatement = $this->m_objPDO->prepare($strQuery); 
        $objStatement->bindParam(':id_role', $pobjRole->getID(), PDO::PARAM_INT);
        return $this->getArrayIDName($objStatement);   
    }
   
    /**
    * Vérifie si l'utilisateur a le doit sur l'action de la ressource demander
    * 
    * @param string                            $pstrUsername  Utilisateur
    * @param com_borealis_acl_object_Ressource $pobjRessource Ressource
    * @param com_borealis_acl_object_Action    $pobjAction    Action
    * 
    * @return bool true or false
    */
    public function isAuthorized($pstrUsername,com_borealis_acl_object_Ressource $pobjRessource,com_borealis_acl_object_Action $pobjAction)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        
        $strQuery = com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectAuthorized();
        $objStatement = $this->m_objPDO->prepare($strQuery); 
        $objStatement->bindParam(':username', $pstrUsername, PDO::PARAM_STR); 
        $objStatement->bindParam(':id_ressource', $pobjRessource->getID(), PDO::PARAM_STR);
        $objStatement->bindParam(':id_action', $pobjAction->getID(), PDO::PARAM_STR);
        self::execute($objStatement); 
        return $objStatement->rowCount() > 0;        
    }
    
    // --------------------------------------------------------------------------
    // --- Private static
    // --------------------------------------------------------------------------
    
    /**
    * Return id and name of all row in the result query
    * 
    * @param PDOStatement &$pobjStatement Statement
    * 
    * @return ArrayObject
    */
    private function getArrayIDName(PDOStatement &$pobjStatement)
    {    
        self::execute($pobjStatement);
        
        $objArrayResult = new ArrayObject();
        while ($objArrayRow = $pobjStatement->fetch(PDO::FETCH_ASSOC)) 
        {
            $objArrayResult->append(array('id' => $objArrayRow['id'], 'name' => $objArrayRow['name']));
        }        
        return $objArrayResult;  
    }
    
    
    /**
    * Return query by type   
    * 
    * @param com_borealis_acl_object_Base $pobjBase  Object
    * @param string                       $pstrQuery Query Name 
    *  
    * @return string
    */
    private static function getQueryByObject(com_borealis_acl_object_Base $pobjBase, $pstrQuery)
    {
        $strType = 'Role';
        if ($pobjBase instanceof com_borealis_acl_object_Action)
        {
            $strType = 'Action';     
        }
        else if ($pobjBase instanceof com_borealis_acl_object_Role)
        {
            $strType = 'Role';     
        } 
        else if ($pobjBase instanceof com_borealis_acl_object_Ressource)
        {
            $strType = 'Ressource';     
        } 
        return self::getQueryByType($strType, $pstrQuery);  
    }
    
    /**
    * Return query by type
    * 
    * @param string $pstrType  Object Type
    * @param string $pstrQuery Query Name
    * 
    * @return string
    */
    private static function getQueryByType($pstrType, $pstrQuery)
    {
        $objArrayQuery = array(
                              'Action'  => array('PREPARE_INSERT'          => com_borealis_acl_SettingImpl::getPDODataStorePrepareInsertAction(),
                                                  'PREPARE_DELETE_BY_ID'   => com_borealis_acl_SettingImpl::getPDODataStorePrepareDeleteActionByID(),
                                                  'PREPARE_UPDATE'         => com_borealis_acl_SettingImpl::getPDODataStorePrepareUpdateAction(),
                                                  'PREPARE_SELECT_BY_NAME' => com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectActionByName(),
                                                  'PREPARE_SELECT_BY_ID'   => com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectActionByID(),
                                                  'PREPARE_SELECT_ALL'     => com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectActionAll()
                                                 ),
                                                 
                              'Role'    => array('PREPARE_INSERT'          => com_borealis_acl_SettingImpl::getPDODataStorePrepareInsertRole(),
                                                  'PREPARE_DELETE_BY_ID'   => com_borealis_acl_SettingImpl::getPDODataStorePrepareDeleteRoleByID(),
                                                  'PREPARE_UPDATE'         => com_borealis_acl_SettingImpl::getPDODataStorePrepareUpdateRole(),
                                                  'PREPARE_SELECT_BY_NAME' => com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRoleByName(),
                                                  'PREPARE_SELECT_BY_ID'   => com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRoleByID(),
                                                  'PREPARE_SELECT_ALL'     => com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRoleAll() 
                                                 ),
                                                 
                              'Ressource'=> array('PREPARE_INSERT'         => com_borealis_acl_SettingImpl::getPDODataStorePrepareInsertRessource(),
                                                  'PREPARE_DELETE_BY_ID'   => com_borealis_acl_SettingImpl::getPDODataStorePrepareDeleteRessourceByID(),
                                                  'PREPARE_UPDATE'         => com_borealis_acl_SettingImpl::getPDODataStorePrepareUpdateRessource(),
                                                  'PREPARE_SELECT_BY_NAME' => com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRessourceByName(),
                                                  'PREPARE_SELECT_BY_ID'   => com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRessourceByID(),
                                                  'PREPARE_SELECT_ALL'     => com_borealis_acl_SettingImpl::getPDODataStorePrepareSelectRessourceAll()
                                                 )
                              );  
          
        return $objArrayQuery[$pstrType][$pstrQuery];      
    }
    
    /**
    * Execute statement and throw exception on error
    * 
    * @param PDOStatement &$pobjStatement PDO Statement already prepare
    * 
    * @throws com_borealis_acl_exception_SQLException
    * 
    * @return void 
    */
    private static function execute(PDOStatement &$pobjStatement)
    {
        if (!$pobjStatement->execute())
        {
            $objArrayErrorInfo = $pobjStatement->errorInfo();
            throw new com_borealis_acl_exception_SQLException("SQLSTATE: $objArrayErrorInfo[0] | Driver Error Code: $objArrayErrorInfo[1] | Description: $objArrayErrorInfo[2]");
        }      
    }
    
}
  
?>
