<?php
/**
* Method utilitaire pour les paths de fichier  
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-foundation  
* @author    Pascal Martin 
* @copyright Boral Informations Stratgiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/
 
/**
* Method utilitaire pour les paths de fichier  
* 
* @category   PHP 
* @package    Borealis-foundation
* @subpackage Util
* @author     Pascal Martin 
*/
class com_borealis_foundation_util_Path
{
    /**
    * Retourne le path absolut
    * Essaie de supprimer les .. et . dans un path
    * 
    * <code>
    *     echo com_borealis_foundation_util_Path::getAbsolute('C:\test\test.php')."\n";
    *     echo com_borealis_foundation_util_Path::getAbsolute('C:/test/test.php')."\n";
    *     echo com_borealis_foundation_util_Path::getAbsolute('C:\test\..\test.php')."\n";
    *     echo com_borealis_foundation_util_Path::getAbsolute('test\..\test.php')."\n";
    *     echo com_borealis_foundation_util_Path::getAbsolute('test/../../test.php')."\n";
    *     echo com_borealis_foundation_util_Path::getAbsolute('test/../.././test.php')."\n";
    *     echo com_borealis_foundation_util_Path::getAbsolute('C:\test1////test2.php')."\n";
    *     echo com_borealis_foundation_util_Path::getAbsolute('\test1////test2.php')."\n";
    * 
    *     // Output
    *     //  C:\test\test.php
    *     //  C:\test\test.php
    *     //  C:\test.php
    *     //  test.php
    *     //  ..\test.php
    *     //  ..\test.php
    *     //  C:\test1\test2.php
    *     //  \test1\test2.php
    * </code>
    * 
    * @param string $pstrPath Path to convert to absolute
    * 
    * @return string
    */
    function getAbsolute($pstrPath) 
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrPath);
        
        $pstrPath = trim($pstrPath);
        $pstrPath = str_replace(array('/', '\\'), DIRECTORY_SEPARATOR, $pstrPath);
        $objArrayParts = array_filter(explode(DIRECTORY_SEPARATOR, $pstrPath), 'strlen');
        $objArrayAbsolutes = array();
        foreach ($objArrayParts as $strPart)
        {
            if ('.' == $strPart)
            {
                continue;
            }
            else if ('..' == $strPart && count($objArrayAbsolutes) > 0)
            {
                array_pop($objArrayAbsolutes);
            }
            else 
            {
                $objArrayAbsolutes[] = $strPart;
            }
        }
        if (substr($pstrPath, 0, 1) == DIRECTORY_SEPARATOR)
        {
            // Linux path 
            return DIRECTORY_SEPARATOR.implode(DIRECTORY_SEPARATOR, $objArrayAbsolutes); 
        }
        else
        {
            // Windows path
            return implode(DIRECTORY_SEPARATOR, $objArrayAbsolutes);  
        } 
    }
}
  
?>
