<?php
/**
* Fichier de configuration du package Borealis-security
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-security  
* @author    Pascal Martin 
* @copyright Boral Informations Stratgiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

/**
* Fichier de configuration du package Borealis-security 
* 
* @category PHP 
* @package  Borealis-security
* @author   Pascal Martin 
*/ 
class com_borealis_security_Authentication
{
    // @codeCoverageIgnoreStart To dificult to test because of use of header
    /**
    * Authentification du user avec la passerelle
    * 
    * @return void
    */
    public static function withGateway()
    {
        // -----------------------------------------------
        // --- Variable global
        // -----------------------------------------------
        $strError = '';    
        $objUser = null; 

        // -----------------------------------------------
        // --- Process Action
        // -----------------------------------------------
        $strError = self::processAction('josso');
        
        // -----------------------------------------------
        // --- Get user
        // -----------------------------------------------
        try
        {
            $objUser = com_borealis_security_manager_SecurityManager::getInstance()->getUser(); 
        }
        catch(com_borealis_security_exception_BadSessionException $objError)
        {
            $strError .= 'Session was close';  
        }  
        
        if ($objUser == null)
        {
            com_borealis_security_manager_SecurityManager::getInstance()->loginWithGateway();
            try
            {
                $objUser = com_borealis_security_manager_SecurityManager::getInstance()->getUser(); 
            }
            catch(Exception $objError)
            {
                // ne rien faire  
            }
            if ($objUser == null)
            {
                echo 'The login gateway is bussy';
                die();   
            }
        }         
    }
    // @codeCoverageIgnoreEnd
    
    /**
    * Authentification du user avec une page de login gnrer par smarty
    * 
    * @param Smarty                                         &$pobjSmarty         Object smarty
    * @param string                                         $pstrTemplate        template de login
    * @param com_borealis_security_password_PasswordCreator $pobjPasswordCreator Createur d'instance password
    * @param bool                                           $pbDie               die() after smarty display
    * 
    * @return void
    */
    public static function withSmarty(Smarty &$pobjSmarty, $pstrTemplate, com_borealis_security_password_PasswordCreator $pobjPasswordCreator, $pbDie = true)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrTemplate);
        
        // -----------------------------------------------
        // --- Variable global
        // -----------------------------------------------
        $strError = '';    
        $objUser = null; 

        // -----------------------------------------------
        // --- Process Action
        // -----------------------------------------------
        $strError = self::processAction('xml', $pobjPasswordCreator);
        
        // -----------------------------------------------
        // --- Get user
        // -----------------------------------------------
        try
        {
            $objUser = com_borealis_security_manager_SecurityManager::getInstance()->getUser(); 
        }
        catch(com_borealis_security_exception_BadSessionException $objError)
        {
            $strError .= 'Session was close';  
        }   
        
        // -----------------------------------------------
        // --- Smarty display
        // -----------------------------------------------
        if ($objUser === null)
        {
            $pobjSmarty->assign('error', $strError);            
            $pobjSmarty->display($pstrTemplate);
            
            // @codeCoverageIgnoreStart
            if ($pbDie === true)
            {            
                die();    
            }
            // @codeCoverageIgnoreEnd
        } 
    }
    
    /**
    * Process action receive by $_GET
    * 
    * @param string                                         $pstrSessionType     Type de session
    * @param com_borealis_security_password_PasswordCreator $pobjPasswordCreator Createur d'instance password
    * 
    * @return string Error message
    */
    private static function processAction($pstrSessionType, com_borealis_security_password_PasswordCreator $pobjPasswordCreator = null)
    {
        $strError = '';
        if (isset($_REQUEST['login']))
        {
            // ------> Login 
            try
            {
                // @codeCoverageIgnoreStart 
                if (isset($_REQUEST['sessiontoken']) && $_REQUEST['sessiontoken'] != '' && $pstrSessionType == 'josso')
                {
                    com_borealis_security_manager_SecurityManager::getInstance()->loginWithSSOSession(com_borealis_security_session_JossoSSOSession::fromToken($_REQUEST['username'], $_REQUEST['sessiontoken']));
                }
                // @codeCoverageIgnoreEnd
                else if (isset($_REQUEST['sessiontoken']) && $_REQUEST['sessiontoken'] != '' && $pstrSessionType == 'xml')
                {
                    com_borealis_security_manager_SecurityManager::getInstance()->loginWithSSOSession(com_borealis_security_session_XMLSSOSession::fromToken($_REQUEST['sessiontoken']));
                }
                else if (isset($_REQUEST['username']) && isset($_REQUEST['password']))
                { 
                    com_borealis_security_manager_SecurityManager::getInstance()->login($_REQUEST['username'], $pobjPasswordCreator->getPassword($_REQUEST['password']));      
                }
            }
            catch(com_borealis_security_exception_BadSessionException $objError)
            {
                $strError .= 'Session was close or bad SSO Session ID';  
            }
            catch(com_borealis_security_exception_UnknownUserException $objError)
            {
                $strError .= 'Bad username or password';
            }
            catch(com_borealis_foundation_exception_AssertException $objError)
            {
                $strError .= 'Username and password ar mandatory';
            }
        }
        else if (isset($_REQUEST['logout']))
        {
            // ------> Logout
            try
            {
                com_borealis_security_manager_SecurityManager::getInstance()->logout();      
            }
            catch(com_borealis_security_exception_BadSessionException $objError)
            {
                $strError .= 'Session was close or was expired';  
            }
        }
        return $strError;
    } 
}

?>