<?php
/**
* Interface pour les diffrentes implmentation de AuthenticationManager 
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-security  
* @author    Pascal Martin 
* @copyright Boral Informations Stratgiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/
    
/**
* Interface pour les diffrentes implmentation de AuthenticationManager
* 
* @category   PHP 
* @package    Borealis-security
* @subpackage Manager
* @author     Pascal Martin
*/
interface com_borealis_security_manager_authentication_AuthenticationManager extends com_borealis_foundation_util_Singleton
{
    /**
    * Authentifie un user et retourne un object session qui reprsente l'utilisateur
    * 
    * @param string                                  $pstrUsername Nom d'utilisateur
    * @param com_borealis_security_password_Password $pobjPassword Mode de passe
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * @throws com_borealis_security_exception_UnknownUserException
    * 
    * @return com_borealis_security_session_Session
    */
    public function login($pstrUsername, com_borealis_security_password_Password $pobjPassword);
    
    /**
    * Authentification du user avec une session dja ouverte
    * 
    * @param com_borealis_security_session_SSOSession $pobjSSOSession Open session 
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * @throws com_borealis_security_exception_BadSessionException
    * 
    * @return com_borealis_security_session_Session
    */
    public function loginWithSSOSession(com_borealis_security_session_SSOSession $pobjSSOSession);
    
    /**
    * Log un utilisateur avec la passerelle
    * 
    * @return com_borealis_security_session_Session   
    */
    public function loginWithGateway(); 
    
    /**
    * Fermeture de la session du user 
    * 
    * @throws com_borealis_security_exception_HostNotFoundException
    * @throws com_borealis_security_exception_ServiceNotFoundException
    * 
    * @return void
    */
    public function logout();
    
    /**
    * Return current open Session or null if no session is open 
    * 
    * @return com_borealis_security_session_Session
    */
    public function getSession();    
    
}
    
    
?>