<?php
/**
* Manager qui sert a la fois d'AuthenticationManager, d'AuthorizationManager et de PropertyManager 
* Rcupre les informations d'un fichier xml.
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-security  
* @author    Pascal Martin 
* @copyright Boral Informations Stratgiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/
    
/**
* Manager qui sert a la fois d'AuthenticationManager, d'AuthorizationManager et de PropertyManager 
* Rcupre les informations d'un fichier xml.
* 
* @category   PHP 
* @package    Borealis-security
* @subpackage Manager
* @author     Pascal Martin
*/
class com_borealis_security_manager_impl_xml_XMLManager implements com_borealis_security_manager_authentication_AuthenticationManager, com_borealis_security_manager_authorization_AuthorizationManager, com_borealis_security_manager_property_PropertyManager
{
    private static $s_objInstance = null;
    
    /**
    * @var com_borealis_security_session_XMLSSOSession
    */
    protected $m_objSSOSession = null;
    
    /**
    * @var SimpleXMLElement
    */
    protected $m_objSimpleXMLElement = null; 
 
    /**
    * @var string
    */
    private $m_strConfigFile = null; 
 
 
    /**
    * Construit une nouvelle instance de com_borealis_security_manager_impl_josso_JossoManager
    * 
    * @return com_borealis_security_manager_impl_josso_JossoManager
    */
    private function __construct()
    {    
        $this->m_objSSOSession = self::getSSOSessionInStore();
        $this->load();
    }
    
    /**
    * return instanceof com_borealis_security_manager_impl_xml_XMLManager
    * return instanceof com_borealis_security_manager_authentication_AuthenticationManager
    * return instanceof com_borealis_security_manager_authorization_AuthorizationManager
    * return instanceof com_borealis_security_manager_property_PropertyManager
    * return instanceof com_borealis_security_util_Singleton
    * 
    * @return com_borealis_security_manager_impl_xml_XMLManager
    */
    public static function getInstance()
    {
        if (self::$s_objInstance === null)
        {
            self::$s_objInstance = new com_borealis_security_manager_impl_xml_XMLManager();
        }
        self::$s_objInstance->checkReload();
        return self::$s_objInstance;
    } 
    
    /**
    * Vrifie si le fichier de configuration xml est plus le mne alors on charge le nouveau fichier
    * 
    * @return void
    */
    public function checkReload()
    {
        if ($this->m_strConfigFile != com_borealis_security_SettingImpl::getXMLConfigFile())
        {
            $this->load();    
        }      
    }
    
    /**
    * Load la configuration xml
    * 
    * @return void
    */
    private function load()
    {
        $this->m_strConfigFile = com_borealis_security_SettingImpl::getXMLConfigFile();
        try
        {
            $this->m_objSimpleXMLElement = simplexml_load_file($this->m_strConfigFile);        
        }
        catch(Exception $objException)
        {
            throw new com_borealis_security_exception_BadXMLConfigFileException($objException, $this->m_strConfigFile);   
        }
    }
    
    // --------------------------------------------------------------------
    // com_borealis_security_manager_authentication_AuthenticationManager
    // --------------------------------------------------------------------
    
    /**
    * Try authentication of the user
    * 
    * @param string                                  $pstrUsername Nom d'utilisateur
    * @param com_borealis_security_password_Password $pobjPassword Mot de passe 
    * 
    * @return com_borealis_security_session_XMLSSOSession
    */
    public function login($pstrUsername, com_borealis_security_password_Password $pobjPassword)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrUsername);
        
        if ($this->getSession() !== null)
        {
            $this->logout();
        }
        
        $strPassword = $pobjPassword->getValue();
        $objArrayResult = $this->m_objSimpleXMLElement->xpath("/security/users/user[./username='$pstrUsername' and ./password='$strPassword']");
        if (count($objArrayResult) == 0)
        {
            throw new com_borealis_security_exception_UnknownUserException('User or password are incorect');   
        }
    
        $objSession = new com_borealis_security_session_XMLSSOSession($pstrUsername, md5($pstrUsername.$strPassword));
        
        $this->setSession($objSession);
        
        return $this->getSession();
    }
    
    /**
    * put your comment there...
    * 
    * @param com_borealis_security_session_SSOSession $pobjSSOSession Open session
    * 
    * @return com_borealis_security_session_XMLSSOSession
    */
    public function loginWithSSOSession(com_borealis_security_session_SSOSession $pobjSSOSession)
    {
        if (!($pobjSSOSession instanceof com_borealis_security_session_XMLSSOSession))
        {
            throw new Exception("not supported");
        }  
        
        if ($this->getSession() !== null && $this->getSession() != $pobjSSOSession)
        {
            $this->logout();
        }        
        $this->setSession($pobjSSOSession);  
        return $this->getSession();       
    }
    
    /**
    * Log un utilisateur avec la passerelle
    * 
    * @return Exception Not supported by xml manager 
    */
    public function loginWithGateway()
    {
        throw new Exception('Not supported by xml manager'); 
    }
    
    /**
    * Clear SSOSession
    * 
    * @return void
    */
    public function logout()
    {
        $this->setSession(null);  
    }
    
    /**
    * Return current SSOSession if user are authenticate 
    * 
    * @return com_borealis_security_session_JossoSSOSession
    */
    public function getSession()
    {
        return $this->m_objSSOSession; 
    }
    
    // --------------------------------------------------------------------
    // com_borealis_security_manager_authorization_AuthorizationManager
    // --------------------------------------------------------------------
    
    /**
    * Set current session
    * 
    * @param com_borealis_security_session_Session $pobjSession Session
    * 
    * @return void
    */
    private function setSession(com_borealis_security_session_Session $pobjSession = null)
    {
        $this->m_objSSOSession = $pobjSession;   
        $this->setSSOSessionInStore($pobjSession);
    }
    
    /**
    * Return list of role for the session
    * 
    * @param com_borealis_security_session_Session $pobjSession target session
    * 
    * @return ArrayObject
    */
    public function getRoles(com_borealis_security_session_Session $pobjSession)
    {
        if (!($pobjSession instanceof com_borealis_security_session_XMLSSOSession))
        {
            throw new Exception("not supported");
        }
        $objSimpleXMLElementUser = $this->getUserNode($pobjSession);
        
        if ($objSimpleXMLElementUser === null)
        {
            throw new com_borealis_security_exception_BadSessionException('Bad session');   
        }
        
        $objArrayRoles = new ArrayObject();
        
        foreach ($objSimpleXMLElementUser->roles->role as $strRole)
        {
            $objArrayRoles->append(new com_borealis_security_user_Role((string)$strRole));   
        }
        return $objArrayRoles;
    }
    
    // --------------------------------------------------------------------
    // com_borealis_security_manager_property_PropertyManager
    // --------------------------------------------------------------------
    
    /**
    * Return list of property for the session
    * 
    * @param com_borealis_security_session_Session $pobjSession target session
    * 
    * @return ArrayObject of com_borealis_security_user_Property
    */
    public function getProperties(com_borealis_security_session_Session $pobjSession)
    {
        if (!($pobjSession instanceof com_borealis_security_session_XMLSSOSession))
        {
            throw new Exception("not supported");
        }
        $objSimpleXMLElementUser = $this->getUserNode($pobjSession);
        
        if ($objSimpleXMLElementUser === null)
        {
            throw new com_borealis_security_exception_BadSessionException('Bad session');   
        }
        
        $objArrayRoles = new ArrayObject();
        
        foreach ($objSimpleXMLElementUser->properties->property as $objSimpleXMLElementProperty)
        {
            $objArrayRoles->append(new com_borealis_security_user_Property((string)$objSimpleXMLElementProperty->name, (string)$objSimpleXMLElementProperty->value));   
        }
        return $objArrayRoles;
    } 
    
    // --------------------------------------------------------------------
    // XML manager method
    // -------------------------------------------------------------------- 
    
    /**
    * Return current logged user node
    * 
    * @param com_borealis_security_session_XMLSSOSession $pobjSession Session  
    * 
    * @return SimpleXMLElement
    */
    private function getUserNode(com_borealis_security_session_XMLSSOSession $pobjSession)
    {
        foreach ($this->m_objSimpleXMLElement->users->user as $objSimpleXMLElementUser)
        {
            $strIdent = md5((string)$objSimpleXMLElementUser->username.(string)$objSimpleXMLElementUser->password);
            if ($strIdent == $pobjSession->getToken())
            {
                return $objSimpleXMLElementUser;
            }   
        }

        return null;
    }
    
    /**
    * Return SSOSession store in a cookie
    * 
    * @return com_borealis_security_session_XMLSSOSession
    */
    private static function getSSOSessionInStore()
    {
        // @codeCoverageIgnoreStart
        if (!isset($_COOKIE['XML_SSOSESSION']))
        {        
            self::setSSOSessionInStore(null);                  
        }
        // @codeCoverageIgnoreEnd
        return unserialize($_COOKIE['XML_SSOSESSION']);     
    } 
    
    /**
    * Store SSOSession in cookie 
    * 
    * @param com_borealis_security_session_XMLSSOSession $pstrSSOSession Session to store
    * 
    * @return void
    */
    private static function setSSOSessionInStore(com_borealis_security_session_XMLSSOSession $pstrSSOSession = null)
    {   
        $_COOKIE['XML_SSOSESSION'] = serialize($pstrSSOSession);  
        setcookie('XML_SSOSESSION', $_COOKIE['XML_SSOSESSION'], 0, "/"); // session cookie ...             
    } 
}
    
    
?>