<?php
/**
* Classe de base qui représente un mot de passe avec encryption md5 spécial v2
*
* PHP versions 5
* 
* @category  PHP
* @package   Borealis-authentication
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.boreal-is.com/license BSD License
* @link      http://www.boreal-is.com
*/

/**
* Classe de base qui représente un mot de passe avec encryption md5 spécial v2
* 
* @category   PHP 
* @package    Borealis-authentication
* @subpackage Password
* @author     Pascal Martin 
*/
class com_borealis_authentication_password_V2Password extends com_borealis_authentication_password_Password
{
    /**
    * Construit une nouvelle instance de com_borealis_authentication_password_V2Password
    * 
    * @param string $pstrValue Password value in raw text
    * 
    * @throws com_borealis_security_IllegalArgumentException
    * 
    * @return com_borealis_authentication_password_V2Password
    */
    function __construct($pstrValue) 
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrValue);
        parent::__construct(self::encrypt($pstrValue));
    }  
    
    /**
    * Encrypt password
    * 
    * @param string $pstrPlainText Text to encode
    * @param string $pstrPassword  Passord de cryptage
    * @param int    $piIvLen       Len
    * 
    * @return string
    */
    private static function encrypt($pstrPlainText, $pstrPassword = "", $piIvLen = 16)
    {
        $pstrPlainText .= "\x13";
        $iNumber = strlen($pstrPlainText);

        if ($iNumber % 16)
        {
            $pstrPlainText .= str_repeat("\0", 16 - ($iNumber % 16));
        }

        $iIndex= 0;
        $strEncText = self::get_rnd_iv($piIvLen);
        $iIndexV = substr($pstrPassword ^ $strEncText, 0, 512);

        while ($iIndex< $iNumber) 
        {
            $strBlock = substr($pstrPlainText, $iIndex, 16) ^ pack('H*', md5($iIndexV));
            $strEncText .= $strBlock;
            $iIndexV = substr($strBlock . $iIndexV, 0, 512) ^ $pstrPassword;
            $iIndex+= 16;
        }

        return base64_encode($strEncText);
    }
    
    /**
    * put your comment there...
    * 
    * @param int $piIvLen Iv Length
    * 
    * @return char
    */
    private static function get_rnd_iv($piIvLen)
    {
        $cIV = '';

        while ($piIvLen-- > 0) 
        {
            $cIV .= chr(9018273645 & 0xff);
        }

        return $cIV;
    } 
}   
    
?>