<?php
/**
* Fichier php pour le l'affichage de la section de la matrice de permission d'un role
*
* PHP versions 5
* 
* @category  PHP
* @package   ACL
* @author    Pascal Martin 
* @copyright Boréal Informations Stratégiques Inc.
* @license   http://www.boreal-is.com/license Borealis License
* @link      http://www.boreal-is.com
*/

// -----------------------------------------------
// --- Include
// -----------------------------------------------
require_once 'common.php'; 

// -----------------------------------------------
// --- Set domain
// -----------------------------------------------
bindtextdomain("rolesmatrix", $g_strLanguageDir);
textdomain("rolesmatrix"); 
bind_textdomain_codeset("rolesmatrix", "UTF-8");

// -----------------------------------------------
// --- Variable global
// -----------------------------------------------
$g_objRole = null; 
$g_objArrayRolePermission = array(); 

// -----------------------------------------------
// --- Action
// -----------------------------------------------

/**
* Construit l'array $g_objArrayRolePermission qui est utiliser par le tpl 
* 
* @return void
*/
function roleACL()
{
    global $g_objRole;
    global $g_objArrayRolePermission;
    
    $g_objRole = com_borealis_acl_object_Role::getByName(trim($_REQUEST['role']));
    $objArrayActions = com_borealis_acl_object_Action::getAll();
    $objArrayRessources = com_borealis_acl_object_Ressource::getAll();
    
    $objArrayTemplate =  array();
    foreach ($objArrayActions as $objAction)
    {
        $strActionName = $objAction->getName();
        $objArrayTemplate[$strActionName] = false;       
    }
    
    foreach ($objArrayRessources as $objRessource)
    {
        $g_objArrayRolePermission[$objRessource->getName()] = array( 'right'   => $objArrayTemplate, 
                                                                     'enabled' => $objArrayTemplate,
                                                                     'resource' => $objRessource); 
        foreach ($objRessource->getActions() as $objAction)
        {
            $g_objArrayRolePermission[$objRessource->getName()]['enabled'][$objAction->getName()] = true;
        }   
    }
    
    foreach ($g_objRole->getPermission() as $objArrayPermission)
    {
        $objRessource = $objArrayPermission['ressource'];
        $objAction = $objArrayPermission['action'];

        $g_objArrayRolePermission[$objRessource->getName()]['right'][$objAction->getName()] = true;               
    }       
}

/**
* Enregistre les modifications aux permission 
* 
* @return void
*/ 
function saveacl()
{
    global $g_objRole;
    $g_objRole = com_borealis_acl_object_Role::getByName(trim($_REQUEST['role']));
    $objArrayPermissions = (array)$g_objRole->getPermission();
    foreach ($_REQUEST['authorized'] as $strRessource => $objArrayActions)
    {
        $objRessource = com_borealis_acl_object_Ressource::getByName($strRessource);
        foreach ($objArrayActions as $strAction => $strValue)
        {
            $objAction = com_borealis_acl_object_Action::getByName($strAction);
            $objArrayPermission = array( 'ressource' => $objRessource, 'action' => $objAction );
            if ($strValue == 'add')
            {
                if (!in_array($objArrayPermission, $objArrayPermissions))
                {
                    $g_objRole->addPermission($objRessource, $objAction);          
                }
            }
            else if ($strValue == 'remove')
            {
                if (in_array($objArrayPermission, $objArrayPermissions))
                {
                    $g_objRole->removePermission($objRessource, $objAction); 
                } 
            }    
        }
    }
}
    
if (isset($_REQUEST['action']))
{   
    switch($_REQUEST['action'])
    {
        case 'saveacl':
            saveacl();
        case 'roleacl':
            roleACL();
            break;        
    }
}

// -----------------------------------------------
// --- Smarty display
// -----------------------------------------------            
$g_objSmarty->assign('roles', com_borealis_acl_object_Role::getAll());             
$g_objSmarty->assign('actions', com_borealis_acl_object_Action::getAll());
$g_objSmarty->assign('role', $g_objRole); 
$g_objSmarty->assign('objArrayRolePermission', $g_objArrayRolePermission);                                                  
                                                                                                                                        
$g_objSmarty->display('rolesmatrix.tpl');
  
?>