<?xml version="1.0" encoding="utf-8"?>
<!-- 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license.
 --> 

<xsl:stylesheet version="1.0" xmlns:php="http://php.net/xsl"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:dyn="http://exslt.org/dynamic"
    xmlns:featurepanel_form="http://geoprisma.org/featurepanel_form"
    extension-element-prefixes="dyn">
  <xsl:namespace-alias stylesheet-prefix="php" result-prefix="xsl" />

<!-- GLOBAL VARIABLES -->
  <!-- todo: currently supports only ONE service type... -->
  <xsl:variable name="pServiceType">
    <xsl:text>featureserver</xsl:text>
  </xsl:variable>

<!-- 
     Function : printWidgetSource

     Print the source code for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="featurepanel_form:printWidgetSource">
    <xsl:variable name="WidgetType">
      <xsl:text>featurepanel_form</xsl:text>
    </xsl:variable>

    <script>
        <xsl:attribute name="src" >
             <xsl:value-of select="$g_widgets_url"/>
             <xsl:text>/featurepanel/form/Form.js</xsl:text>
        </xsl:attribute>
    </script>

    <!-- Traduction (utilisation de la mÃ©thode extjs)
                http://extjs.com/learn/Tutorial:Localizing_Ext#Application_Localization
        --> 
    <script>            
        Ext.apply(org.GeoPrisma.FeaturePanel.Form.prototype, {
            i18n_form_button_commit_text:    <xsl:value-of select="php:functionString('FeaturePanel_Form::getText','i18n_form_button_commit_text','JS')"/>,            
            i18n_form_button_commit_tooltip: <xsl:value-of select="php:functionString('FeaturePanel_Form::getText','i18n_form_button_commit_tooltip','JS')"/>,            
            i18n_form_button_cancel_text:    <xsl:value-of select="php:functionString('FeaturePanel_Form::getText','i18n_form_button_cancel_text','JS')"/>,            
            i18n_form_button_cancel_tooltip: <xsl:value-of select="php:functionString('FeaturePanel_Form::getText','i18n_form_button_cancel_tooltip','JS')"/>           
        });  
    </script>
    
    <script type="text/javascript">
      function objGP<xsl:value-of select='$WidgetType'/>GetFieldByName(szName, objFields){
          for(var i=0, len=objFields.length; i&lt;len; i++ ){
              var objField = objFields[i];
              if(objField['name'] == szName){
                  return objField;
              }
          }
          return false;
      };
    </script>
  </xsl:template>

<!-- 
     Function : printWidgetExecution

     Print the execution code for this widget
-->
  <!-- Call for each widget name-->
  <xsl:template name="featurepanel_form:printWidgetExecution">
    <xsl:param name="pWidgetName" />
    <xsl:param name="pMapName" />

    <xsl:variable name="WidgetType">
      <xsl:text>featurepanel_form</xsl:text>
    </xsl:variable>

    <script type="text/javascript">
      var objItems = {};
      var aResources = [];


      <!-- widget OPTIONS object used to create the widget class -->
      <!-- N.B. <fields> gets also parsed.  It's useless since it's treated
           seperately to create Ext.form objects before creating the actual
           widget, but it has no effects it's kept that way...
        -->
      var objWidgetOptions =
      <xsl:for-each select="./options">
        <xsl:call-template name="getObjectFromNode">
          <xsl:with-param name="pNode" select="." />
        </xsl:call-template>
      </xsl:for-each>;

      <!-- Traduction du title -->
      <xsl:if test="./options/windowoptions/title">
        objWidgetOptions['windowoptions']['title'] = <xsl:for-each select="./options/windowoptions/title" ><xsl:call-template name="getText" /></xsl:for-each>;
      </xsl:if>          
      
      <!-- Traduction des fieldLabel -->  
      <xsl:for-each select="./options/fields/field">  
          <xsl:variable name="index" select="position()"/>
          <xsl:if test="./fieldLabel">
            objWidgetOptions['fields'][<xsl:value-of select="$index - 1" />]['fieldLabel'] = <xsl:for-each select="./fieldLabel" ><xsl:call-template name="getText" /></xsl:for-each>;
          </xsl:if>
      </xsl:for-each>                   
      
      <xsl:for-each select="./resources/resource">
        <!-- ResourceName -->
        <xsl:variable name="pResourceName">
          <xsl:value-of select="." />
        </xsl:variable>

        <!-- ServiceName -->
        <xsl:variable name="pServiceName">
          <xsl:call-template name="getDataStoreInfoFromWidget">
            <xsl:with-param name="pWidgetName" select="$pWidgetName" />
            <xsl:with-param name="pServiceType" select="$pServiceType" />
            <xsl:with-param name="pNodeName"><xsl:text>service</xsl:text></xsl:with-param>
            <xsl:with-param name="pResourceName" select="$pResourceName"/>
          </xsl:call-template>
        </xsl:variable>

        <!-- DataStoreName  -->
        <xsl:variable name="pDataStoreName">
          <xsl:call-template name="getDataStoreInfoFromWidget">
            <xsl:with-param name="pWidgetName" select="$pWidgetName" />
            <xsl:with-param name="pServiceType" select="$pServiceType" />
            <xsl:with-param name="pNodeName"><xsl:text>name</xsl:text></xsl:with-param>
            <xsl:with-param name="pResourceName" select="$pResourceName"/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:for-each select="../../../../datastores/datastore[./name = $pDataStoreName]">
            var objFields = [];
            var szDisplayField = 'text';
            var szValueField = 'value';
            var objDisplayValueFields = [szValueField,szDisplayField];
            var objStore, szDefaultValue;

            <!-- create one javascript object of ExtJS form options per field-->
            <!-- TODO: change params for layer -->
            <xsl:for-each select="./params/fields/field">
              <!-- field 'text' if any, else take value instead-->
              <xsl:variable name="fieldText">
	        <xsl:choose>
                  <xsl:when test="./text">
                    <xsl:for-each select="./text" ><xsl:call-template name="getText" /></xsl:for-each>
		  </xsl:when>
		  <xsl:otherwise>
                    '<xsl:value-of select="./value" />'
		  </xsl:otherwise>
	        </xsl:choose>
              </xsl:variable>

              objStore = null;
              szDefaultValue = null;
              objData = [];

              <!-- If there are values set in the datastore, get them  -->
              <xsl:for-each select="./values/value">
                <!-- field 'text' if any, else take value instead-->
                <xsl:variable name="valueText">
	          <xsl:choose>
                    <xsl:when test="./text">
                      <xsl:for-each select="./text" ><xsl:call-template name="getText" /></xsl:for-each>
		    </xsl:when>
		    <xsl:otherwise>
                      '<xsl:value-of select="./value" />'
		    </xsl:otherwise>
	          </xsl:choose>
                </xsl:variable>

                objData.push(['<xsl:value-of select="./value" />',<xsl:value-of select="$valueText" />]);
	      </xsl:for-each>

              <xsl:if test="./values">
                  objStore = new Ext.data.SimpleStore({
                      fields: objDisplayValueFields, data : objData });
                  szDefaultValue = objStore.data.items[0].data.value;
              </xsl:if>

              <xsl:variable name="fieldOptions">
                <xsl:if test="./values">
                  store: objStore,
                  value: szDefaultValue,
                  valueField: szValueField,
                  displayField: szDisplayField,
                </xsl:if>
              </xsl:variable>              

              <!-- Create an options object with key:value of properties found
                   in the datastore
                -->
              objFields.push({
                  <xsl:value-of select="$fieldOptions" />
                  name : '<xsl:value-of select="./name" />',
                  fieldLabel: <xsl:value-of select="$fieldText" />
              });          
            </xsl:for-each> <!-- end of for-each fields in datastore-->
          </xsl:for-each> <!-- end of for-each datastore of the resource -->

          <!-- Now that the options object are created, we need to determine
               the xtype of the form element to create, add the default option
               values then the options defined in the widget.
            -->

          <!-- THIS is the array of field objects that will be added to the
               ext form object
            -->
          objItems = [];

          <xsl:for-each select="./../../options/fields/field">
            //objField = getFieldByName(<xsl:value-of select="./name" />);
            objField = objGP<xsl:value-of select='$WidgetType'/>GetFieldByName('<xsl:value-of select="./name" />', objFields);

            if(objField){
              <!-- add the options defined in the <field> node of the
                   widget.
                -->
               objWidgetFieldOptions = 
                 <xsl:call-template name="getObjectFromNode">
                   <xsl:with-param name="pNode" select="." />
                 </xsl:call-template>;
              
              <!-- Traduction du fieldLabel -->
              <xsl:if test="./fieldLabel">
                objWidgetFieldOptions['fieldLabel'] = <xsl:for-each select="./fieldLabel" ><xsl:call-template name="getText" /></xsl:for-each>;
              </xsl:if>            
                 
               objField = OpenLayers.Util.extend(objField, objWidgetFieldOptions);
               objItems.push(objField);
            }
          </xsl:for-each>

          <!-- TODO: test with 2 resources  -->

          objWidgetOptions['mappanelid'] = 'gp<xsl:value-of select="$pMapName" />';
          objWidgetOptions['items'] = objItems;
          objGPWidget<xsl:value-of select="$pWidgetName" /> = new org.GeoPrisma.FeaturePanel.Form(objWidgetOptions);

          <!-- Find each widget that contains this featurepanel_form and 
               add it using the addFeaturePanel function -->
          <xsl:for-each select="//options[./featurepanel = $pWidgetName]">
            <xsl:for-each select="../name">
              <xsl:variable name="ownerWidgetName">
                <xsl:value-of select="." /><xsl:value-of select="$pResourceName" />
              </xsl:variable>

              objGPWidget<xsl:value-of select="$ownerWidgetName" />.setFeaturePanel(objGPWidget<xsl:value-of select="$pWidgetName" />);
            </xsl:for-each>
          </xsl:for-each>

      </xsl:for-each><!-- end of for-each resources -->

    </script>
  </xsl:template>

<!-- 
     Function : drawWidget

     Output the widget depending on the render type
-->
  <!-- Call for each widget name -->
  <xsl:template name="featurepanel_form:drawWidget">
    <xsl:param name="pWidgetName" />
      objGP<xsl:value-of select="$pWidgetName" />
  </xsl:template>

<!--
    Function: getWidgetDepedencies

    Return the list of dependencies for this widget
-->
  <!-- Call for each widget type-->
  <xsl:template name="featurepanel_form:getWidgetDepedencies">
<!-- TODO : this widget has a toolbar,map,Result.js and queryonclick dependency -->
  </xsl:template>

</xsl:stylesheet>
