<?php
/* 
   Copyright (c) 2009- Boreal - Information Strategies, published under the BSD license.
   See http://geoprisma.org/license for the full text of the license. 
*/ 
  
class ResultExtGrid extends org_geoprisma_widget_WidgetBase
{
    static public function getText($pstrMsg, $pstrType = null)
    {
        return parent::getText($pstrMsg, self::getType(), $pstrType);
    }

    static public function getAction()
    {
        return 'read';
    }    
     
    static public function getType()
    {
        return 'resultextgrid';
    }

    static public function getServiceType()
    {
        return 'wms';
    }

    static public function getMandatoryResourceOptionList()
    {
        return array("result");
    }

    static public function getOptionalResourceOptionList()
    {
        return null;
    }

    static public function mustClone()
    {
        return false;
    }

    static public function getMandatoryResourceOptionValueList() 
    {
        return array("result" => "resultextgrid");
    }

    static public function linkResource($pobjResource, $pobjWidget) 
    {
        if ($pobjResource == null || $pobjWidget == null) 
        {
            return;
        }

        // if the widget has the "useResponseFields" option set to true,
        // then no need to create any grid at all
        try
        {
            if ($pobjWidget->getOption('useResponseFields')->getValue() === "true") 
            {
                return;
            }
        }
        catch (Exception $objError) {}

        $objArrayGrids = null;

        try
        {
            $objArrayGrids = $pobjWidget->getOption('grids')->getValue();
        }
        catch(Exception $objError)
        {
            $pobjWidget->addOrReplaceOption(
                new org_geoprisma_option_Option('grids', array())
            );
            $objArrayGrids = $pobjWidget->getOption('grids')->getValue();
        }

        $objArrayFields = $pobjResource->getFields();
        if ($objArrayFields && $objArrayFields->count() > 0) 
        {
            $objFields = array();

            foreach ($pobjResource->getFields() as $objField) 
            {
                array_push($objFields, self::getFieldNode($objField));
            }
            
            array_push(
                $objArrayGrids,
                array(
                    'resourcename' => $pobjResource->getName(),
                    'displayname' => $pobjResource->getTitle(),
                    'fields' => $objFields
                )
            );
        }
        
        $pobjWidget->addOrReplaceOption(
            new org_geoprisma_option_Option('grids', $objArrayGrids)
        );
    }

    static public function getFieldNode($pobjField) 
    {
        $objArrayFieldNode = array(
            'id' => $pobjField->getName(),
            'label' => $pobjField->getTitle()
        );

        foreach ($pobjField->getOptions() as $objOption) 
        {
            $objArrayFieldNode[$objOption->getName()] = $objOption->getValue();
        }

        return $objArrayFieldNode;
    }
}
  
?>
