GeoExt.ux.FeatureGridResult = Ext.extend( Ext.grid.GridPanel, {
    position_tooltip_text : "Go to position",
    

    /**
     * Property: featureConfigs (Array) An array of all the features added
     */
    featureConfigs: [],

    /**
     * Property: selectedIndex (int) The index of the selected resource 
     * in the featureConfigs array
     */
    selectedIndex: null,

    /**
     * APIProperty: GeoExt.ux.WFSFilterBuilderButton
     */
    wfsFilterBuilderButton: null,

    /**
     * APIProperty: map (Object) The default map of the application
     */
    map: null,

    /**
     * APIProperty: maxZoom
     * {Integer} The closest zoom level to use when action is triggered
     */
    maxZoom: null,

    /**
     * APIProperty: minZoom
     * {Integer} The minimum zoom level to use when action is triggered
     */
    minZoom: null,

    /**
     * Property: enableHdMenu
     * {Bool} enable or disable list header menu
     */
    enableHdMenu: false,
    
    /**
     * Method: initComponent
     * This method initialize the actions of the component
     */
    initComponent: function() {
        //Listen the rowdblclick action
        this.on("rowdblclick", function(grid, rowIndex, event) {
            this.showWindowData(rowIndex);
        }, this);
        
        GeoExt.ux.FeatureGridResult.superclass.initComponent.apply(this, arguments);  
    },

    /**
     * Method: adaptColumnsToStore
     * 
     * The method adapt the columns to the selected store
     * 
     * Parameters: store - (gxp.data.WFSFeatureStore) The store used for
     * the panel columns - (Array) The array that contain columns
     */
    adaptColumnsToStore : function() {

        var featureConfigs = this.featureConfigs[this.selectedIndex];
        var store = featureConfigs.store;
        var columns = featureConfigs.columns;
        var newColumns = [];

        // (1) add a button for the 'method' defined, if any
        featureConfigs.options.method && newColumns.push({
            header : '',
            xtype : 'actioncolumn',
            width : 32,
            items : [{
                iconCls : featureConfigs.options.selectorClass,
                tooltip : featureConfigs.options.selectorTooltip,
                handler : function(grid, rowIndex, colIndex) {
                    this.showWindowData(rowIndex);
                },
                scope: this
            }] 
        });

        // (2) add recenter button
        featureConfigs.noLayer === false && newColumns.push({
            header : '',
            xtype : 'actioncolumn',
            width : 32,
            items : [ {
                iconCls : 'gx-wfsfilterbuilder-featuregridresult-position-button',
                tooltip : this.position_tooltip_text,
                handler : function(grid, rowIndex, colIndex) {
                    var feature = grid.getStore().getAt(rowIndex).getFeature();
                    
                    // find zoom level and center
                    var bounds = feature.geometry.getBounds();
                    var center = bounds.getCenterLonLat();
                    // code for wrapDateLine open layer option
                    if (this.map.baseLayer.wrapDateLine) {
                        var maxExtent = this.map.getMaxExtent();
                        bounds = bounds.clone();
                        while (bounds.right < bounds.left) {
                            bounds.right += maxExtent.getWidth();
                        }
                        center = bounds.getCenterLonLat().wrapDateLine(maxExtent);
                    }
                    
                    var zoomToExtent = this.map.getZoomForExtent(bounds);
                    
                    // if zoom level is greather than max zoom level, replace it
                    if (this.maxZoom !== null && zoomToExtent > this.maxZoom) {
                        zoomToExtent = this.maxZoom;
                    }
                    // if zoom level is lower than min zoom level, replace it
                    else if (this.minZoom !== null && zoomToExtent < this.minZoom) {
                        zoomToExtent = this.minZoom;
                    }
                    
                    // apply Center and zoom level
                    this.map.setCenter(center, zoomToExtent);
                    
                    this.wfsFilterBuilderButton.hideWindowOnRecenter &&
                        // Close/Hide all Ext.Window currently visible
                        Ext.WindowMgr.each(function(win){
                            if (win.isVisible()) {
                                if (win.closeAction == "hide" ) {
                                    win.hide();
                                } else {
                                    win.close();
                                }
                            }
                        }, this);
                },
                scope: this
            } ] 
        });

        // (3) add fields, skip ignored ones
        for ( var i = 0; i < columns.length; i++) {
            column = columns[i];
            if (featureConfigs.ignoredFields.indexOf(column.header) == -1) {
                newColumns.push(column); 
            }
        }

        // Apply the column and store to the grid
        var cm = new Ext.grid.ColumnModel(newColumns);
        this.reconfigure(store, cm);
    },


    /**
     * Method: createAction
     * 
     * This method show a new window and call the method in the options of 
     * the resource to fill the new window
     * 
     * Parameters: 
     * store - (int) The index of the selected resource
     * rowIndex - (int) The index of the selected row
     */
    showWindowData: function(rowIndex) {
        var config = this.featureConfigs[this.selectedIndex];
        var record = config.store.getAt(rowIndex);
        var id = record.get(config.options.primeryField);
        var resourceName = config.resource;
        new Ext.Window({
            items: config.options.method(resourceName, id),
            width: config.selectorTemplateWidth || 1200,
            height: config.selectorTemplateHeight || 600,
            layout: "fit",
            closable : true,
            border : false,
            modal: true,
            plain    : true,
            resizable : false,
            autoScroll: true,
            constrain: true,
            region: 'center'
        }).show();
    }
    
});


