<?php
/**
* Abstract class for each config elements.
*
* PHP version 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Alexandre Dube
* @copyright 2010, Nippour Geomatik
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/
 
/**
* Abstract class for each config elements.
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage ConfigElement 
* @author     Alexandre Dube
* @license    http://www.geoprisma.org/license BSD License
* @link       http://www.geoprisma.org
*/ 

abstract class org_geoprisma_ConfigElement
{
    /**
    * Return DomElement object of given value.
    *
    * @param string      $pstrType        Type
    * @param string      $pstrValue       Value
    * @param DOMDocument $pobjDomDocument DomDocument object used to create 
    *                                     dom elements
    * 
    * @return array
    */
    public function getDomElements(
        $pstrType, $pstrValue, DOMDocument $pobjDomDocument
    ) {
        $objDomElement = null;

        // support for org_geoprisma_option_Option values
        if (is_object($pstrValue)
            && get_class($pstrValue) == "org_geoprisma_option_Option"
        ) 
        {
            $pstrType = $pstrValue->getName();
            $pstrValue = $pstrValue->getValue();
        }
        
        if (is_array($pstrValue) || is_object($pstrValue)) 
        {
            $strSubType = null;

            if (strtolower(substr($pstrType, -1)) == 's') 
            {
                $strSubType = substr($pstrType, 0, -1);
            }

            $objDomElement = $pobjDomDocument->createElement($pstrType);
            foreach ($pstrValue as $strArrayKey => $strArrayValue) 
            {
                if (is_numeric($strArrayKey) && $strSubType) 
                {
                    $objChildDomElement = $this->getDomElements(
                        $strSubType, $strArrayValue, $pobjDomDocument
                    );
                } else {
                    $objChildDomElement = $this->getDomElements(
                        $strArrayKey, $strArrayValue, $pobjDomDocument
                    );
                }

                $objDomElement->appendChild($objChildDomElement);
            }
        } else {
            $objDomElement = $pobjDomDocument->createElement(
                $pstrType, $pstrValue
            );
        }

        return $objDomElement;
    }

    /**
    * Given an array of records (that were fetched from sql requests, xml to 
    *     array conversion, etc.), validate that the elements are correctly
    *     formated.
    *
    * DEPRECATED
    *
    * @param array $pobjArray The config element array
    * 
    * @return array
    */
    public function getConfigElements(array $pobjArray) 
    {
        $pobjArrayMainKeys = array_keys($pobjArray);
        $pobjArrayValues = $pobjArray[$pobjArrayMainKeys[0]];

        if (count($pobjArrayValues) == 1 && is_array($pobjArrayValues)) 
        {
            $objArrayKeys = array_keys($pobjArrayValues);

            if (!is_int($objArrayKeys[0])
                && is_array($pobjArrayValues[$objArrayKeys[0]])
            ) 
            {
                $objArrayChildKeys = array_keys($pobjArrayValues[$objArrayKeys[0]]);
                if (is_int($objArrayChildKeys[0])) 
                {
                    $pobjArray[$pobjArrayMainKeys[0]]
                        = $pobjArrayValues[$objArrayKeys[0]];
                }
            }
        }

        return $pobjArray;
    }
}
  
?>
