<?php
/**
* Get configuration from database using the "MapContext" way of defining it.
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Alexandre Dube
* @copyright 2010, Nippour Geomatik
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Get configuration from database using the "MapContext" way of defining it.
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Config  
* @author     Alexandre Dube
*/ 
class org_geoprisma_config_PGSQLMapContextConfig extends org_geoprisma_config_Config
{
    /**
    * @var org_geoprisma_config_PGSQLMapContextConfig
    */
    private static $s_objInstance = null;

    /**
    * @var boolean Whether the driver should create (MapPanel and Layer) OR
    *              (Map and MapFishLayerTree) widgets
    */
    private static $s_bUseLegacyWidgets = false;
    
    protected $m_objArrayService = null;
    protected $m_objArrayLayout = null;
    protected $m_objArrayDatastore = null;
    protected $m_objArrayWidget = null;
    protected $m_objArrayAccessFilter = null;
    protected $m_objArrayResource = null;
    protected $m_objArrayOtherResource = null;
    protected $m_objArrayField = null;

    /**
    * @var array
    */
    protected $m_objArrayClonedWidgetNames = null;

    /**
    * @var string
    */
    private static $s_strConnection = null;

    /**
    * @var object
    */
    private $m_objDB = null;

    /**
    * @var integer
    */
    private static $s_iCurrentSession = -1;
    private static $s_iCurrentMapContext = -1;
    private static $s_iCurrentApplication = -1;

    /**
    * @var string
    */
    private static $s_strCurrentSessionName = "";
    private static $s_strCurrentMapContextName = "";
    private static $s_strCurrentApplicationName = "";

    /**
    * @var string
    */
    private static $s_strServiceFeature = "service";
    private static $s_strDataStoreFeature = "datastore";
    private static $s_strResourceFeature = "resource";
    private static $s_strFieldFeature = "field";
    private static $s_strMapContextFeature = "mapcontext";
    private static $s_strWidgetFeature = "widget";
    private static $s_strAccessFilterFeature = "accessfilter";
    private static $s_strApplicationFeature = "application";
    private static $s_strSessionFeature = "session";

    private static $s_strTablePrefix = "mcconfig_";
    private static $s_strTableOptionSuffix = "option";

    /**
    * @var array
    *
    * Only used if isUsingLegacyWidgets() != true
    */
    protected $m_objArrayLayerResourceOptions = array(
        "visibility" => "visibility",
        "maxscaledenom" => "minscale",
        "minscaledenom" => "maxscale"
    );

    protected $m_objArrayProcessList = array(
        "getProcessList",
        "getAuthorizedResources"
    );
    
    /**
    * @var array
    */
    protected $m_objArrayServiceTypes = array(
        1 => org_geoprisma_service_WMSService::TYPE,
        2 => org_geoprisma_service_TileCacheService::TYPE,
        3 => org_geoprisma_service_FeatureServerService::TYPE,
        4 => org_geoprisma_service_GYMOService::TYPE,
        5 => org_geoprisma_service_WidgetService::TYPE,
        6 => org_geoprisma_service_MapServerService::TYPE,
        7 => org_geoprisma_service_MapFishPrintService::TYPE,
        8 => org_geoprisma_service_FileService::TYPE,
        9 => org_geoprisma_service_WFSService::TYPE,
        10 => org_geoprisma_service_HttpRequestService::TYPE,
        11 => org_geoprisma_service_WPSService::TYPE,
        12 => org_geoprisma_service_TMSService::TYPE
    );

    /**
    * @var array
    */
    protected $m_objArrayApplicationTypes = array(
        1 => "extjs"
    );

    /**
    * @var array
    */
    protected $m_objArrayWidgetTypes = array(
        2 => "map",
        3 => "mapfishlayertree",
        4 => "mapfishrecenter",
        5 => "measuretool",
        7 => "queryonclick",
        9 => "resultextgrid",
        11 => "initialview",
        12 => "zoomslider",
        13 => "mouseposition",
        14 => "getmouseposition",
        15 => "scale",
        16 => "editfeature_create",
        17 => "editfeature_update",
        18 => "editfeature_delete",
        19 => "featurepanel_form",
        20 => "featurepanel_selector",
        22 => "geoexttoolbar",
        25 => "shortcut",
        26 => "geoextux_geonamessearchcombo",
        27 => "geoextux_redliningpanel",
        28 => "pdfprint",
        29 => "wmslayeradder",
        30 => "geoextux_shortcutcombo",
        31 => "geoextux_zoomto",
        32 => "htwindow",
        33 => "unselectall",
        34 => "geoextprintform",
        35 => "filetreepanel",
        36 => "geoextux_layertreebuilder",
        37 => "mappanel",
        38 => "layer",
        39 => "querybyrect",
        40 => "featurepanel_attributeform",
        41 => "featurepanel_customform",
        42 => "geoextux_wmsbrowser",
        43 => "vectorlayer",
        44 => "resultvectorlayer",
        45 => "queryonclickwizard",
        46 => "geoextux_printpreview",
        47 => "wfsfilterbuilder",
        48 => "templatepopup",
        49 => "toggle",
        50 => "editfeature_split",
        51 => "editfeature_drag",
        52 => "editfeature_copy",
        53 => "editfeature_confirm",
        54 => "keepactivesession",
        55 => "applyfilter",
        // for user-defined widgets, use numbers grater than 9999 ??
        10000 => "livemarker",
        10666 => "multiprojectionmouseposition"

    );


    /**
    * Construct a new instance of org_geoprisma_config_PGSQLMapContextConfig.
    * Force the use of 'getInstance' (Singleton)
    * 
    * @throws org_geoprisma_exception_BadPGSQLMapContextConfig
    * 
    * @return org_geoprisma_config_PGSQLMapContextConfig
    */
    private function __construct()
    {
        $this->m_objArrayService = new ArrayObject();
        $this->m_objArrayDatastore = new ArrayObject();
        $this->m_objArrayResource = new ArrayObject();
        $this->m_objArrayOtherResource = new ArrayObject();
        $this->m_objArrayWidget = new ArrayObject();
        $this->m_objArrayExclusiveUnboundWidgetName = array();
        $this->m_objArrayAccessFilter = new ArrayObject();
        $this->m_objArrayLayout = new ArrayObject();
        $this->m_objArrayField = new ArrayObject();

        $this->m_objArrayClonedWidgetNames = array();

        $this->load();
    }

    // ====================
    //  connection methods
    // ====================

    /**
    * Connect to database using connection string.
    * 
    * @return void
    */
    protected function connect()
    {
        if ($this->m_objDB) 
        {
            return;
        }

        $this->m_objDB = pg_connect($this->getConnectionString()); 

        if (!$this->m_objDB) 
        {
            throw new Exception("Unable to connect to the database");
        }
    }

    /**
    * Disconnect from database.
    * 
    * @return void
    */
    protected function disconnect()
    {
        pg_close($this->m_objDB);
        $this->m_objDB = null;
    }

    // ==============
    //  LOAD methods
    // ==============

    /**
    * Get config from database.
    * 
    * @return void
    */
    protected function load()
    {
        try
        {
            // connect to db
            $this->connect();

            // First, check if there is/are 'osmresource' parameter(s) in the
            // url, meaning a specific list of Resources accessed
            $objArrayResourceNames = $this->getResourceNamesFromRequest();
            if (count($objArrayResourceNames) > 0) 
            {
                $this->loadFromResourceList(
                    $objArrayResourceNames, array('loadFields' => false)
                );
            }
            // Else, that means we load the whole config.  Check if we load from
            // it from a Session id (not PHP Session, but GeoPrisma session)
            else if (self::getCurrentSession() != -1
                || self::getCurrentSessionName() != ""
            ) 
            {
                $this->loadFromSession();
            }
            // Else, check if we load from MapContext + Application combinaison
            else if ((self::getCurrentMapContext() != -1 || self::getCurrentMapContextName() != "")
                && (self::getCurrentApplication() != -1 || self::getCurrentApplicationName() != "") 
            ) 
            {
                $this->loadFromMapContext();
            }
            // well, there is nothing to load from...
            else
            {
                exit("no session OR mapcontext+application defined");
            }
            
            $this->disconnect();

            // test
            //print_r($this->getAuthorizedResources());

            //print_r($this);
            //exit("\nok");
        }
        catch(Exception $objError)
        {
            throw new org_geoprisma_exception_BadPGSQLMapContextConfigException($objError->__toString());       
        }  
    }
    
    /**
    * Using current 'Session' id or name, get the according 'MapContext' and
    *     'Application' ids and load them.
    * 
    * @return void
    */
    protected function loadFromSession()
    {
        if (self::getCurrentSession() != -1) 
        {
            $objArrayOptions = array(
                'singleRecord' => self::getCurrentSession()
            );
        }
        else
        {
            $objArrayOptions = array(
                'singleRecordName' => self::getCurrentSessionName()
            );
        }

        $objSession = $this->getRecords(
            $this->getSessionFeatureName(),
            $objArrayOptions
        );

        if (self::getCurrentSession() == -1) 
        {
            self::setCurrentSession($objSession['id']);
        }

        self::setCurrentMapContext(
            $objSession[$this->getMapContextFeatureName().'_id']
        );
        self::setCurrentApplication(
            $objSession[$this->getApplicationFeatureName().'_id']
        );

        $this->loadFromMapContext();
    }

    /**
    * Using current 'MapContext' id, load : 
    *     Resources, DataStores, Services, Fields and MapWidget.
    * 
    * @return void
    */
    protected function loadFromMapContext()
    {
        // get mapcontext with resource ids
        $strForeignFeature = $this->getResourceFeatureName();
        $objArrayOptions = array(
            'foreignFeature' => $strForeignFeature,
            'getOptions' => true
        );

        if (self::getCurrentMapContext() != -1) 
        {
            $objArrayOptions['singleRecord'] = self::getCurrentMapContext();
        }
        else
        {
            $objArrayOptions['singleRecordName']
                = self::getCurrentMapContextName();
        }

        $objMapContext = $this->getRecords(
            $this->getMapContextFeatureName(),
            $objArrayOptions
        );

        if (self::getCurrentMapContext() == -1) 
        {
            self::setCurrentMapContext($objMapContext['id']);
        }

        // get resource ids array
        $objArrayResourceIds = $this->getForeignIds(
            array($objMapContext), $strForeignFeature
        );

        // load service, datastore, fields and resources from resource ids list
        $this->loadFromResourceList($objArrayResourceIds);

        // load layout and widgets
        $this->loadFromApplication();

        if ($this->isUsingLegacyWidgets()) 
        {
            // load map widget from mapcontext
            $this->loadMapWidget($objMapContext);
        }
        else
        {
            // load mappanel and layer widgets from mapcontext
            $this->loadMapPanelWidget($objMapContext);
            $this->loadLayerWidgets($objMapContext);
        }
    }

    /**
    * Using a resource ids array, load : Resources, DataStores, Services and
    *     (optionnally) Fields.
    * 
    * @param array $pobjArrayResourceIds A resource id list
    * @param array $pobjArrayOptions     Options for this method.
    * 
    * @return void
    */
    protected function loadFromResourceList(
        $pobjArrayResourceIds,
        $pobjArrayOptions = array('loadFields' => true, 'otherResources' => false)
    ) {
        $bLoadFields = $pobjArrayOptions['loadFields'];
        if (isset($pobjArrayOptions['otherResources'])) 
        {
            $bOtherResources = $pobjArrayOptions['otherResources'];
        }
        else
        {
            $bOtherResources = false;
        }

        // get resources with datastore ids
        $objArrayForeignFeatures = array($this->getDataStoreFeatureName());
        array_push($objArrayForeignFeatures, $this->getAccessFilterFeatureName());
        if ($bLoadFields) 
        {
            array_push($objArrayForeignFeatures, $this->getFieldFeatureName());
        }

        $objArrayOptions = array(
            'specificRecords' => $pobjArrayResourceIds,
            'foreignFeatures' => $objArrayForeignFeatures,
            'getOptions' => true
        );

        if (self::getCurrentMapContext() != -1) 
        {
            $objArrayOptions['orderingTable']
                = $this->getMapContextFeatureName();
            $objArrayOptions['orderingMasterFeatureId']
                = self::getCurrentMapContext();
        }

        if ($bLoadFields) 
        {
            $objArrayOptions['foreignOrdering'] = array(
                $this->getFieldFeatureName()
            );
        }

        $objArrayResources = $this->getRecords(
            $this->getResourceFeatureName(),
            $objArrayOptions
        );

        // get datastore ids array
        $objArrayDatastoreIds = $this->getForeignIds(
            $objArrayResources, $objArrayForeignFeatures[0]
        );

        if ($bLoadFields) 
        {
            // get fields ids array
            $objArrayFieldIds = $this->getForeignIds(
                $objArrayResources, $objArrayForeignFeatures[2]
            );

            // get fields
            $objArrayOptions = array(
                'specificRecords' => $objArrayFieldIds,
                'getOptions' => true,
                'orderingTable' => $this->getResourceFeatureName()
            );
            $objArrayFields = $this->getRecords(
                $this->getFieldFeatureName(),
                $objArrayOptions
            );
        }

        // get datastores
        $strForeignFeature = $this->getServiceFeatureName();
        $objArrayOptions = array(
            'specificRecords' => $objArrayDatastoreIds,
            'getOptions' => true
        );
        $objArrayDataStores = $this->getRecords(
            $this->getDataStoreFeatureName(),
            $objArrayOptions
        );

        // get service ids array
        $objArrayServiceIds = $this->getForeignIds(
            $objArrayDataStores, $strForeignFeature
        );

        // get services
        $objArrayOptions = array(
            'specificRecords' => $objArrayServiceIds,
            'getOptions' => true
        );
        $objArrayServices = $this->getRecords(
            $this->getServiceFeatureName(),
            $objArrayOptions
        );

        // get accessfilters
        $objArrayAccessFilterIds = $this->getForeignIds(
            $objArrayResources, $objArrayForeignFeatures[1]
        );

        $objArrayOptions = array(
            'specificRecords' => $objArrayAccessFilterIds,
            'getOptions' => true
        );
        $objArrayAccessFilters = $this->getRecords(
            $this->getAccessFilterFeatureName(),
            $objArrayOptions
        );

        // ======
        //  load
        // ======

        $this->loadServices($objArrayServices);
        $this->loadDatastores($objArrayDataStores);
        $this->loadAccessFilters($objArrayAccessFilters);
        if ($bLoadFields) 
        {
            $this->loadFields($objArrayFields);
        }
        $this->loadResources(
            $objArrayResources,
            Array('otherResources' => $bOtherResources)
        );
    }

    /**
    * Using current 'Application' id, load : Layout and Widgets
    * 
    * @return void
    */
    protected function loadFromApplication()
    {
        // get application with widget ids
        $strForeignFeature = $this->getWidgetFeatureName();
        $objArrayOptions = array(
            'foreignFeature' => $strForeignFeature
        );

        if (self::getCurrentApplication() != -1) 
        {
            $objArrayOptions['singleRecord'] = self::getCurrentApplication();
        }
        else
        {
            $objArrayOptions['singleRecordName']
                = self::getCurrentApplicationName();
        }

        $objApplication = $this->getRecords(
            $this->getApplicationFeatureName(),
            $objArrayOptions
        );

        if (self::getCurrentApplication() == -1) 
        {
            self::setCurrentApplication($objApplication['id']);
        }

        // get widgets ids array
        $objArrayWidgetIds = $this->getForeignIds(
            array($objApplication), $strForeignFeature
        );

        // get widgets
        $objArrayOptions = array(
            'specificRecords' => $objArrayWidgetIds,
            'getOptions' => true,
            'optionOrdering' => true
        );

        if (self::getCurrentApplication() != -1) 
        {
            $objArrayOptions['orderingTable']
                = $this->getApplicationFeatureName();
            $objArrayOptions['orderingMasterFeatureId']
                = self::getCurrentApplication();
        }

        $objArrayWidgets = $this->getRecords(
            $this->getWidgetFeatureName(),
            $objArrayOptions
        );

        // ======
        //  load
        // ======

        $this->loadLayout($objApplication);
        $this->loadWidgets($objArrayWidgets);
        $this->adjustWidgetsContainingClonedWidgets();
    }


    /**
    * Load all 'org_geoprisma_service_Service' object inside the service array
    *     from the database.
    * 
    * @param array $pobjArrayServices Array of service records
    * 
    * @return void
    */
    protected function loadServices($pobjArrayServices)
    {
        foreach ($pobjArrayServices as $objService)
        {
            $strType = $this->m_objArrayServiceTypes[$objService['type']];

            $this->m_objArrayService[$objService['id']] = (
                org_geoprisma_service_ServiceFactory::getService(
                    $strType, $objService
                )
            );  
        }
    }
    
    /**
    * Load the 'org_geoprisma_layout_Layout' object using the application 
    * record.
    * 
    * @param array $pobjApplication A single application record.
    * 
    * @return void
    */
    protected function loadLayout($pobjApplication)
    {
        $this->m_objArrayLayout[$pobjApplication['id']] 
            = new org_geoprisma_layout_Layout(
                (string)$pobjApplication['id'],
                (string)$pobjApplication['template'],
                (string)$this->m_objArrayApplicationTypes[$pobjApplication['type']]
        );
    }
   
    /**
    * Load all 'org_geoprisma_datastore_Datastore' objects inside the datastore
    * array using datastore records.
    * 
    * @param array $pobjArrayDatastores Array of datastore records
    * 
    * @return void
    */
    private function loadDatastores($pobjArrayDatastores)
    {
        foreach ($pobjArrayDatastores as $objDatastore)
        {
            $this->m_objArrayDatastore->append(
                new org_geoprisma_datastore_Datastore(
                    $objDatastore['id'],
                    $this->m_objArrayService[$objDatastore[$this->getServiceFeatureName().'_id']],
                    $objDatastore['layers'],
                    $objDatastore['options']
                )
            );
        }
    }

    /**
    * Load all 'org_geoprisma_accessfilter_AccessFilter' objects inside the accessfilter
    * array using accessfilter records.
    * 
    * @param array $pobjArrayAccessFilters Array of accessfilter records
    * 
    * @return void
    */
    private function loadAccessFilters($pobjArrayAccessFilters)
    {
        foreach ($pobjArrayAccessFilters as $objAccessFilter)
        {
            $objArrayConditions = array();
            foreach ($objAccessFilter['options'] as $strName => $strValue) 
            {
                if ($strName == 'condition') 
                {
                    $objArrayConditions[] = $strValue;
                }
            }

            $this->m_objArrayAccessFilter->append(
                new org_geoprisma_accessfilter_AccessFilter(
                    $objAccessFilter['id'],
                    $objArrayConditions
                )
            );
        }
    }

    /**
    * Load all 'org_geoprisma_field_Field' objects inside the field array using
    *     field records.
    * 
    * @param array $pobjArrayFields Array of field records
    * 
    * @return void
    */
    private function loadFields($pobjArrayFields)
    {
        foreach ($pobjArrayFields as $objField)
        {
            $objArrayOption = new ArrayObject();

            // field title (with i18n support)
            if ($objField['key'] != '' && $objField['domain'] != '') 
            {
                $strTitle = array(
                    'key' => $objField['key'],
                    'domain' => $objField['domain']
                );
            }
            else
            {
                $strTitle = (string)$objField['title'];
            }

            $this->m_objArrayField->append(
                new org_geoprisma_field_Field(
                    $objField['id'],
                    $objField['name'],
                    $objField,
                    $strTitle
                )
            );
        }
    }
    
    /**
    * Load all 'org_geoprisma_widget_Widget' objects inside the widget
    * array using widget records.
    * 
    * @param array $pobjArrayWidgets Array of widget records
    * 
    * @return void
    */
    protected function loadWidgets($pobjArrayWidgets)
    {
        foreach ($pobjArrayWidgets as $objWidgetRecord)
        {
            $bAddWidget = false;

            $objWidget = org_geoprisma_widget_WidgetFactory::getWidget(
                $this->m_objArrayWidgetTypes[$objWidgetRecord['type']],
                $objWidgetRecord
            );

            $objArrayResourceOptions
                = $objWidget->getMandatoryResourceOptionList();

            // if widget doesn't need to be linked to any resource, add it.
            if ($objArrayResourceOptions === null) 
            {
                // the name 'linkResource' becomes irrevelant here, but the
                // purpose of this method should remain the same (call before
                // each widget added and do custom stuff with or without a
                // resource)...  If the method returns
                if ($objWidget->linkResource(null, $this) !== false) 
                {
                    // add widget to config only
                    $this->m_objArrayWidget[$objWidget->getName()] = $objWidget;
                }
            }
            else
            {
                foreach ($this->getResources() as $objResource)
                {
                    $bHasRequiredOptions 
                        = $objWidget->resourceHasRequiredWidgetOptions(
                            $objResource
                        );

                    // if resource has all required options used by the widget
                    // and user is authorized to access the resource, add the
                    // the widget and add the resource to it.
                    if ($bHasRequiredOptions
                        && org_geoprisma_acl_ACLFactory::getACL()->isAuthorized($objResource, $objWidget->getAction())
                    ) 
                    {
                        // if widget and resource has a 1:1 relation, then it
                        // must be cloned first.
                        if ($objWidget->mustClone()) 
                        {
                            $objWidgetClone = clone $objWidget;
                            $strOriginalName = $objWidgetClone->getName();
                            $strCloneName = $strOriginalName.'_'.uniqid();
                            $this->addCloneName(
                                $strOriginalName, $strCloneName
                            );

                            $objWidgetClone->setName($strCloneName);

                            $objArrayWidgetResourceOptions 
                                = $objWidgetClone->getOptionsFromResource(
                                    $objResource
                                );

                            foreach ($objArrayWidgetResourceOptions 
                                     as $objOption) 
                            {
                                $objWidgetClone->addOrReplaceOption(
                                    $objOption
                                );
                            }

                            // if clone linked, then add widget to resource and
                            // add it to widget array
                            if ($objWidgetClone->linkResource($objResource, $this) !== false) 
                            {
                                $objResource->addWidget($objWidgetClone);
                                $this->m_objArrayWidget[$objWidgetClone->getName()] = $objWidgetClone;
                            }
                        }
                        // the widget has a 1:x relation with resources, so
                        // we simply need to  link 
                        else
                        {
                            $bAddWidget = true;

                            // if linked, then add widget to resource
                            if ($objWidget->linkResource($objResource, $this) !== false)
                            {
                                $objResource->addWidget($objWidget);
                            }
                        }
                    }
                } // end of foreach resources

                if ($bAddWidget) 
                {
                    $this->m_objArrayWidget[$objWidget->getName()] = $objWidget;
                }

            } // end of 'else'

        } // end of foreach widgets
    }

    /**
    * Add a new clone name in the 'clone' array using the original name as
    *     the key.
    *
    * @param string $pstrOriginalName The original name
    * @param string $pstrCloneName    The name of the clone
    * 
    * @return void
    */
    protected function addCloneName($pstrOriginalName, $pstrCloneName) 
    {
        if (!isset($this->m_objArrayClonedWidgetNames[$pstrOriginalName])) 
        {
            $this->m_objArrayClonedWidgetNames[$pstrOriginalName] = array();
        }

        array_push(
            $this->m_objArrayClonedWidgetNames[$pstrOriginalName],
            $pstrCloneName
        );
    }

    /**
    * This config driver sometimes clones widget and change their name.  The
    *     names that appears in some widgets have to be changed accordingly.  
    *     This is done using the 'getCloneNames' method.
    * 
    * @return void
    */
    protected function adjustWidgetsContainingClonedWidgets() 
    {
        // key : widgettype that can contain reference to cloned widgets
        // value : the node name to check
        $objArrayWidgetsToCheck = array(
            "geoexttoolbar" => "widget",
            "toolbar" => "widget",
            "queryonclick" => "featurepanel"
        );

        foreach ($this->m_objArrayWidget as $objWidget) 
        {
            $strType = $objWidget->getType();

            if (!isset($objArrayWidgetsToCheck[$strType])) 
            {
                continue;
            }

            $strSingleNodeName = $objArrayWidgetsToCheck[$strType];
            $strMultipleNodeName = $strSingleNodeName."s";

            try
            {
                $objOption = $objWidget->getOption($strSingleNodeName);

                $objArrayClonedNames = $this->getCloneNames(
                    $objOption->getValue()
                );

                if ($objArrayClonedNames !== false) 
                {
                    if (count($objArrayClonedNames) == 1) 
                    {
                        $objOption->setValue($objArrayClonedNames[0]);
                    }
                    else
                    {
                        $objOption->setValue($objArrayClonedNames);
                        $objOption->setName($objOption->getName().'s');
                    }
                }
            }
            catch(Exception $objError)
            {
                try
                {
                    $objArrayNewValues = array();

                    $objOption = $objWidget->getOption($strMultipleNodeName);
                    $objArrayWidgetNames = $objOption->getValue();

                    foreach ($objArrayWidgetNames as $strWidgetName) 
                    {
                        $objArrayClonedNames = $this->getCloneNames(
                            $strWidgetName
                        );

                        if ($objArrayClonedNames !== false) 
                        {
                            $objArrayNewValues = array_merge(
                                $objArrayNewValues, $objArrayClonedNames
                            );
                        }
                        else
                        {
                            array_push($objArrayNewValues, $strWidgetName);
                        }
                    }

                    $objOption->setValue($objArrayNewValues);
                }
                catch(Exception $objError) {
                    // no option was found, so carry on...
                }
            } 
        }
    }
    
    /**
    * Load all 'org_geoprisma_resource_Resource' objects inside the resource
    * array using resource records.  Since widgets are no longer directly 
    * attached to resources, they are loaded separately.
    * 
    * @param array $pobjArrayResources Array of resource records
    * @param array $pobjArrayOptions   Array of options for this method
    * 
    * @return void
    */
    protected function loadResources(
        $pobjArrayResources,
        $pobjArrayOptions = array('otherResources' => false)
    ) {
        $bOtherResources = $pobjArrayOptions['otherResources'];
        if ($bOtherResources) 
        {
            $objArrayDestResource = 'm_objArrayOtherResource';
        }
        else
        {
            $objArrayDestResource = 'm_objArrayResource';
        }
        
        foreach ($pobjArrayResources as $objResource) 
        {
            // resource name
            $strName = (string)$objResource['id'];

            // resource acl name
            $strACLName = (string)$objResource['acl_name'];

            // resource title (with i18n support)
            if ($objResource['key'] != '' && $objResource['domain'] != '') 
            {
                $strTitle = array(
                    'key' => $objResource['key'],
                    'domain' => $objResource['domain']
                );
            }
            else
            {
                $strTitle = (string)$objResource['name'];
            }

            $objArrayDatastore = new ArrayObject();
            $objArrayField = new ArrayObject();
            $objArrayWidget = new ArrayObject();
            $objArrayAccessFilter = new ArrayObject();
            $objArrayOption = new ArrayObject();
            
            // get datastores
            if (isset($objResource[$this->getDataStoreFeatureName()])) 
            {
                foreach ($objResource[$this->getDataStoreFeatureName()] as $strDatastore)
                {
                    $objArrayDatastore[$strDatastore] 
                        = $this->getDatastore((string)$strDatastore);
                }
            }

            // get fields (if any)
            if (isset($objResource[$this->getFieldFeatureName()])) 
            {
                foreach ($objResource[$this->getFieldFeatureName()] as $strField)
                {
                    $objArrayField[$strField] 
                        = $this->getField((string)$strField);
                }
            }

            // create options
            foreach ($objResource['options'] 
                     as $strOptionName => $strOptionValue) 
            {
                $objArrayOption->append(
                    new org_geoprisma_option_Option(
                        $strOptionName, $strOptionValue
                    )
                );
            }
            
            // get accessfilters
            if (isset($objResource[$this->getAccessFilterFeatureName()])) 
            {
                foreach ($objResource[$this->getAccessFilterFeatureName()] as $strAccessFilter)
                {
                    $objArrayAccessFilter[$strAccessFilter] 
                        = $this->getAccessFilter((string)$strAccessFilter);
                }
            }
            
            // append new resource
            $this->$objArrayDestResource->append(
                new org_geoprisma_resource_Resource(
                    $strName,
                    $objArrayDatastore,
                    $objArrayWidget,
                    $objArrayAccessFilter,
                    $objArrayOption,
                    $strTitle,
                    $objArrayField,
                    $strACLName
                )
            );
        }
    }

    /**
    * Load all 'org_geoprisma_resource_Resource' objects inside the other resource array.
    * 
    * @return void
    */
    protected function loadOtherResources()
    {
        $objArrayOptions = array(
            //'specificRecords' => $pobjArrayResourceIds,
            'foreignFeatures' => $objArrayForeignFeatures,
            'getOptions' => false
        );
        $this->connect();

        //get les resources.
        $objArrayOtherResources = $this->getRecords(
            $this->getResourceFeatureName(),
            $objArrayOptions
        );
        
        $objArrayOtherResourceIds = Array();
        //créer un array d'id qui sera charger par la fontion loadFromResourceList.
        foreach ($objArrayOtherResources as $objOtherResource) 
        {
            $objArrayOtherResourceIds[] = $objOtherResource['id'];
        }

        $pobjArrayOptions = array('loadFields' => true, 'otherResources' => true);
        $iSavedMapContext = self::getCurrentMapContext();
        self::setCurrentMapContext(-1);

        $this->loadFromResourceList(
            $objArrayOtherResourceIds,
            $pobjArrayOptions
        );

        self::getCurrentMapContext($iSavedMapContext);
        $this->disconnect();
    }
    /**
    * Create the 'mappanel' widget and append it to widget array using the
    * MapContext record.
    *
    * Only used if isUsingLegacyWidgets() != true
    * 
    * @param array $pobjMapContext The MapContext record.
    * 
    * @return void
    */
    protected function loadMapPanelWidget($pobjMapContext)
    {
        $objArrayMapPanel = array(
            'name' => $pobjMapContext['name'],
            'options' => $pobjMapContext['options']
        );
        $objMapPanelWidget = new org_geoprisma_widget_GenericWidget('mappanel', $objArrayMapPanel);

        // add mappanel widget as first element of the widget array
        $this->m_objArrayWidget = new ArrayObject(
            array('mappanel'=>$objMapPanelWidget) +
            (array)$this->m_objArrayWidget
        );
    }

    /**
    * For each resource, create 'layer' widgets according to the datastores
    * they have, link them to and append them to widget array.
    *
    * Only used if isUsingLegacyWidgets() != true
    * 
    * @return void
    */
    protected function loadLayerWidgets()
    {
        $objArrayLayers = array();
        $objArrayResources = $this->getResources();
        $bAddLayer = false;

        foreach ($objArrayResources as $objResource)
        {
            $objArrayResourceOptions = $objResource->getOptions();
            $objArrayDataStores = $objResource->getDatastores();
            $strResourceName = $objResource->getName();

            foreach ($objArrayDataStores as $objDataStore)
            {
                $objService = $objDataStore->getService();
                $strServiceType = $objService->getType();
                $strServiceName = $objService->getName();
                $strDataStoreName = $objDataStore->getName();

                switch ($strServiceType) {
                    // WMS
                    case org_geoprisma_service_WMSService::TYPE:
                        $objTileCacheDS
                            = $objResource->getDatastoreByServiceType(
                                org_geoprisma_service_TileCacheService::TYPE
                            );

                        $objTMSDS
                            = $objResource->getDatastoreByServiceType(
                                org_geoprisma_service_TMSService::TYPE
                            );

                        // add WMS layer if there is no tilecache or tms
                        // datastore
                        $bAddLayer = ($objTileCacheDS || $objTMSDS)
                            ? false : true;
                        
                        break;

                    // TileCache, GYMO and TMS
                    case org_geoprisma_service_TileCacheService::TYPE:
                    case org_geoprisma_service_GYMOService::TYPE:
                    case org_geoprisma_service_TMSService::TYPE:
                        // always add tilecache, gymo and tms layers
                        $bAddLayer = true;
                        break;

                    // FeatureServer & WFS
                    case org_geoprisma_service_FeatureServerService::TYPE:
                    case org_geoprisma_service_WFSService::TYPE:
                        // if there is at least one widget requiring to interact
                        // with a vector layer OR the resource only has one 
                        // datastore, add a 'featureserver' or 'wfs' layer
                        $bLayerNeededByWidgets
                            = $objResource->hasWidgetUsingServiceType(
                                $strServiceType
                            );

                        if ($bLayerNeededByWidgets
                            || $objResource->getDatastores()->count() == 1
                        ) 
                        {
                            $bAddLayer = true;
                        }

                        break;
                }

                if ($bAddLayer) 
                {
                    $objArrayLayerOptions 
                        = $this->getLayerOptionsFromResourceOptions(
                            (array)$objArrayResourceOptions,
                            $strServiceType
                        );

                    $objArrayLayerOptions = array_merge(
                        $objService->getDefaultLayerOptions($this),
                        $objArrayLayerOptions,
                        array('servicetype' => $strServiceType)
                    );

                    $strLayerId = "layer_S".$strServiceName."_DS".
                        $strDataStoreName."_R".$strResourceName;

                    $objLayerWidget = new org_geoprisma_widget_GenericWidget(
                        'layer',
                        array(
                            'id' => $strLayerId,
                            'options' => $objArrayLayerOptions
                        )
                    );

                    $objArrayLayers[$strLayerId] = $objLayerWidget;
                    $objResource->addWidget($objLayerWidget);

                    $bAddLayer = false;
                }
            }
        }

        // add all created layer widgets to the widget array
        // add mappanel widget as first element of the widget array
        $this->m_objArrayWidget = new ArrayObject(
            (array)$this->m_objArrayWidget +
            $objArrayLayers
        );
    }

    /**
    * Load 'org_geoprisma_widget_MapWidget' object and append to widget
    * array using MapContext record.
    * 
    * @param array $pobjMapContext The MapContext record.
    * 
    * @return void
    */
    protected function loadMapWidget($pobjMapContext)
    {
        // create layers
        $objArrayLayers = array();
        $objArrayWMSServiceLayerKeys = array();
        $objArrayResources = $this->getResources();
        foreach ($objArrayResources as $objResource)
        {
            $objArrayResourceOptions = $objResource->getOptions();
            $objArrayDataStores = $objResource->getDatastores();
            $strResourceName = $objResource->getName();

            foreach ($objArrayDataStores as $objDataStore)
            {
                $objService = $objDataStore->getService();
                $strServiceType = $objService->getType();
                $strServiceName = $objService->getName();

                $objArrayLayerOptions 
                    = $this->getLayerOptionsFromResourceOptions(
                        (array)$objArrayResourceOptions,
                        $strServiceType
                    );

                switch ($strServiceType) {
                    // WMS
                    case org_geoprisma_service_WMSService::TYPE:
                        $objTileCacheDS
                            = $objResource->getDatastoreByServiceType(
                                org_geoprisma_service_TileCacheService::TYPE
                            );

                        // if resource has tilecache datastore, don't add a
                        // wms layer.  Remove the node from the layertree as
                        // well
                        if ($objTileCacheDS) 
                        {
                            $objMapFishLayerTree 
                                = $this->getMapFishLayerTreeWidget();
                            if ($objMapFishLayerTree) 
                            {
                                $objMapFishLayerTree->removeLeaf(
                                    $objMapFishLayerTree->getNodes(),
                                    $strResourceName,
                                    $strServiceType
                                );
                            }
                            break;
                        }

                        // check if a layer object with the same service name
                        // has already been created.  If not, create a new layer
                        // and keep the layer+servicename link using an array.
                        if (!isset($objArrayWMSServiceLayerKeys[$strServiceName])) 
                        {
                            $strLayerName = "L_".$strServiceType."_R_".$strResourceName;
                            $objArrayLayerOptions = array_merge(
                                $objService->getDefaultLayerOptions($this),
                                $objArrayLayerOptions,
                                array('layername' => $strLayerName)
                            );
                            array_push(
                                $objArrayLayers, 
                                array(
                                    'resourcenames' => array($strResourceName),
                                    'servicetype' => $strServiceType,
                                    'options' => $objArrayLayerOptions
                                )
                            );

                            // keep the layer+service name link
                            $objArrayWMSServiceLayerKeys[$strServiceName]
                                = count($objArrayLayers) - 1;
                        }
                        else
                        {
                            array_push($objArrayLayers[$objArrayWMSServiceLayerKeys[$strServiceName]]['resourcenames'], $strResourceName);
                            $objArrayLayers[$objArrayWMSServiceLayerKeys[$strServiceName]]['options']['layername'] .= "_".$strResourceName;
                        }
                        break;

                    // TileCache
                    case org_geoprisma_service_TileCacheService::TYPE:
                        $strLayerName = "L_".$strServiceType."_R_".$strResourceName;
                        $objArrayLayerOptions = array_merge(
                            $objService->getDefaultLayerOptions($this),
                            $objArrayLayerOptions,
                            array('layername' => $strLayerName)
                        );
                        array_push(
                            $objArrayLayers,
                            array(
                                'resourcenames' => array($strResourceName),
                                'servicetype' => $strServiceType,
                                'options' => $objArrayLayerOptions
                            )
                        );
                        break;

                    // FeatureServer
                    case org_geoprisma_service_FeatureServerService::TYPE:
                    case org_geoprisma_service_WFSService::TYPE:
                        // if there are no widgets requiring direct 
                        // interaction AND there and resource has more than
                        // one datastore, don't add a 'featureserver' or 'wfs'
                        // layer
                        $bLayerNeededByWidgets
                            = $objResource->hasWidgetUsingServiceType(
                                $strServiceType
                            );

                        if ($bLayerNeededByWidgets
                            || $objResource->getDatastores()->count() == 1
                        ) 
                        {
                            $strLayerName = "L_".$strServiceType."_R_".$strResourceName;
                            $objArrayLayerOptions = array_merge(
                                $objService->getDefaultLayerOptions($this),
                                $objArrayLayerOptions,
                                array('layername' => $strLayerName)
                            );
                            array_push(
                                $objArrayLayers,
                                array(
                                    'resourcenames' => array($strResourceName),
                                    'servicetype' => $strServiceType,
                                    'options' => $objArrayLayerOptions
                                )
                            );
                        }
                        // if the layer is not added, we need to make sure
                        // the according layertree node is removed as well
                        else
                        {
                            $objMapFishLayerTree 
                                = $this->getMapFishLayerTreeWidget();
                            if ($objMapFishLayerTree) 
                            {
                                $objMapFishLayerTree->removeLeaf(
                                    $objMapFishLayerTree->getNodes(),
                                    $strResourceName,
                                    $strServiceType
                                );
                            }
                        }
                        break;

                    // GYMO
                    case org_geoprisma_service_GYMOService::TYPE:
                        $strLayerName = "L_".$strServiceType."_R_".$strResourceName;
                        $objArrayLayerOptions = array_merge(
                            $objService->getDefaultLayerOptions($this),
                            $objArrayLayerOptions,
                            array('layername' => $strLayerName)
                        );
                        array_push(
                            $objArrayLayers,
                            array(
                                'resourcenames' => array($strResourceName),
                                'servicetype' => $strServiceType,
                                'options' => $objArrayLayerOptions
                            )
                        );
                        break;
                }
            }
        }

        // create map with layers
        $objArrayMap = array(
            'name' => $pobjMapContext['name'],
            'options' => $pobjMapContext['options'],
            'layers' => $objArrayLayers
        );

        $this->m_objArrayWidget['map'] = new org_geoprisma_widget_MapWidget($objArrayMap);
    }

    // ==========================================================
    //  getRecords method and utilities (connecting to database)
    // ==========================================================

    /**
    * Get records from specific table.
    *
    * @param sring $pstrFeature      Name of the feature we need to get
    * @param array $pobjArrayOptions Array of options used to fetch more
    *                                elements inside the records
    * 
    * @return void
    */
    protected function getRecords ( $pstrFeature, $pobjArrayOptions=array() )
    {
        // when set to true, get the records options as well
        $bGetOptions = isset($pobjArrayOptions['getOptions']) 
            ? $pobjArrayOptions['getOptions'] : null;

        // merge record options to the record properties OR create a separate
        // 'options' array to put them there
        $bMergeOptions = isset($pobjArrayOptions['mergeOptions'])
            ? $pobjArrayOptions['mergeOptions'] : null;

        // when set, get foreign records as well in defined foreignFeatures
        $objArrayForeignFeatures = isset($pobjArrayOptions['foreignFeatures'])
            ? $pobjArrayOptions['foreignFeatures'] : array();

        // when set, get foreign records as well in defined foreignFeature
        if (isset($pobjArrayOptions['foreignFeature'])) 
        {
            array_push(
                $objArrayForeignFeatures,
                $pobjArrayOptions['foreignFeature']
            );
        }

        // if set, get a single record using a specific id.  Not compatible
        // with 'singleRecordName' nor 'specificRecords'.
        $iSingleRecord = isset($pobjArrayOptions['singleRecord'])
            ? $pobjArrayOptions['singleRecord'] : null;

        // if set, get a single record using a specific name.  Not compatible
        // with 'singleRecord' nor 'specificRecords'.
        $strSingleRecordName = isset($pobjArrayOptions['singleRecordName'])
            ? $pobjArrayOptions['singleRecordName'] : null;

        // if set, get records from the specified list of ids.  Not compatible
        // with 'singleRecordName' nor 'singleRecord'.
        $objArraySpecificRecords = isset($pobjArrayOptions['specificRecords'])
            ? $pobjArrayOptions['specificRecords'] : null;

        // the name of the table related to the feature
        $strTable = $this->getTableName($pstrFeature);

        // the name of the ordering table to use to order the features
        $strOrderTable = null;
        $strOrderMasterFeature = null;
        if (isset($pobjArrayOptions['orderingTable'])) 
        {            
            $strOrderTable = $this->getRelationTableName(
                $pobjArrayOptions['orderingTable'],
                $pstrFeature
            );
            $strOrderMasterFeature = $pobjArrayOptions['orderingTable'];
        }

        // the id of the 'master' feature used when ordering.  Is not mandatory
        // for all kinds of features.
        $iOrderMasterId = null;
        if (isset($pobjArrayOptions['orderingMasterFeatureId'])) 
        {
            $iOrderMasterId = $pobjArrayOptions['orderingMasterFeatureId'];
        }

        // when set to 'true', order the options by 'order'
        $bOptionOrdering = isset($pobjArrayOptions['optionOrdering'])
            ? $pobjArrayOptions['optionOrdering'] : null;

        // when set to 'true', foreign requests are going to be ordered by 
        // 'order'
        $objArrayForeignOrdering = isset($pobjArrayOptions['foreignOrdering']) 
            ? $pobjArrayOptions['foreignOrdering'] : array();

        $bWhere = false;

        $strSQL = 'SELECT '.$strTable.'.* ';
        $strSQL .= ($strOrderTable) ? ", ".$strOrderTable.".order " : "";
        $strSQL .= 'FROM '.$strTable;
        $strSQL .= ($strOrderTable) ? ", ".$strOrderTable : "";
        if ($iSingleRecord) 
        {
            $strSQL .= " WHERE ".$strTable.".id='".$iSingleRecord."'";
            $bWhere = true;
        }
        else if ($objArraySpecificRecords) 
        {
            $objArrayWhere = array();
            for ($iIter=0, $iLen=count($objArraySpecificRecords); $iIter<$iLen; $iIter++)
            {
                $objArrayWhere[$iIter] = $strTable.".id='".$objArraySpecificRecords[$iIter]."'";
            }
            $strSQL .= " WHERE ( ".implode(" OR ", $objArrayWhere)." ) ";
            $bWhere = true;
        }
        else if ($strSingleRecordName) 
        {
            $strSQL .= " WHERE ".$strTable.".name='".$strSingleRecordName."'";
            $bWhere = true;
        }
        
        if ($strOrderTable) 
        {
            $strSQL .= ($bWhere) ? " AND " : " WHERE ";
            $strSQL .= $strOrderTable.'.'.$pstrFeature.'_id = '.
                    $strTable.'.id';
            if ($iOrderMasterId) 
            {
                $strSQL .= " AND ".$strOrderTable.'.'.$strOrderMasterFeature.
                    '_id = '.$iOrderMasterId;
            }
            $bWhere = true;
        }
        $strSQL .= ($strOrderTable) ? ' ORDER BY "order"' : "";

        $objArrayRecords = array();
        $objResult = pg_query($this->m_objDB, $strSQL);

        if ($objResult) 
        {
            // fetch results
            while (($objArrayRow = pg_fetch_assoc($objResult)) !== false)
            {
                $objArrayRecords[$objArrayRow['id']] = $objArrayRow;
                if ($bGetOptions && !$bMergeOptions) 
                {
                    $objArrayRecords[$objArrayRow['id']]['options'] = array();
                }
            }
            pg_free_result($objResult);

            // at least one record is required if asking for a specific one(s)
            if (count($objArrayRecords) == 0 
                && ($iSingleRecord || $objArraySpecificRecords || $strSingleRecordName)
            ) 
            {
                $strMessage = "Failed to fetch the specific \"$pstrFeature\" record(s)";
                throw new Exception($strMessage);
            }

            // if we were using the 'single record name' method, keep track of
            // its id from now on
            if ($strSingleRecordName) 
            {
                $objArrayRecord = end($objArrayRecords);
                $iSingleRecord = $objArrayRecord['id'];
            }

            // get options (optional) START
            if ($bGetOptions) 
            {
                $strOptionTable = $this->getOptionTableName($pstrFeature);

                $objArraySQL = array();
                array_push(
                    $objArraySQL,
                    "SELECT $strOptionTable.* FROM $strTable, $strOptionTable"
                );
                array_push(
                    $objArraySQL,
                    'WHERE '.$strOptionTable.'.'.$pstrFeature.'_id ='
                );
                array_push(
                    $objArraySQL,
                    $strTable.'.id'
                );

                $strSQL = implode(" ", $objArraySQL);

                if ($iSingleRecord) 
                {
                    $strSQL .= " AND ".$pstrFeature."_id='".$iSingleRecord."'";
                }
                else if ($objArraySpecificRecords) 
                {
                    $objArrayWhere = array();
                    for ($iIter=0, $iLen=count($objArraySpecificRecords); $iIter<$iLen; $iIter++)
                    {
                        $objArrayWhere[$iIter] = $pstrFeature."_id='".$objArraySpecificRecords[$iIter]."'";
                    }
                    $strSQL .= " AND ( ".implode(" OR ", $objArrayWhere)." )";
                }

                if ($bOptionOrdering) 
                {
                    $strSQL .= 'ORDER BY '.$strOptionTable.'."order"';
                }

                $objArrayOptions = array();
                $objResult = pg_query($this->m_objDB, $strSQL);
                if ($objResult) 
                {
                    while (($objArrayRow = pg_fetch_assoc($objResult)) !== false)
                    {
                        array_push($objArrayOptions, $objArrayRow);
                        if ($bMergeOptions) 
                        {
                            $objArrayRow['options'] = array();
                        }
                    }
                    pg_free_result($objResult);

                    $iNumRec = count($objArrayOptions);
                    for ($iIter=0; $iIter<$iNumRec; $iIter++)
                    {
                        // if option has 'domain' and 'key' properties set, then
                        // create the option value from these values
                        if (isset($objArrayOptions[$iIter]['domain'])
                            && isset($objArrayOptions[$iIter]['key'])
                            && $objArrayOptions[$iIter]['domain'] != ""
                            && $objArrayOptions[$iIter]['key'] != ""
                        ) 
                        {
                            $objArrayValue = array(
                                'domain' => $objArrayOptions[$iIter]['domain'],
                                'key' => $objArrayOptions[$iIter]['key']
                            );
                        }
                        else
                        {
                            $objArrayValue = $objArrayOptions[$iIter]['value'];
                        }

                        $strOptionName = $objArrayOptions[$iIter]['name'];

                        if (!$bMergeOptions) 
                        {
                            $this->setOptionValue(
                                $objArrayRecords[$objArrayOptions[$iIter][$pstrFeature.'_id']]['options'],
                                $strOptionName,
                                $objArrayValue
                            );
                        }
                        else
                        {
                            $this->setOptionValue(
                                $objArrayRecords[$objArrayOptions[$iIter][$pstrFeature.'_id']],
                                $strOptionName,
                                $objArrayValue
                            );
                        }
                    }
                } 
            }
            // get options (optional) END

            // get associated records from all foreign tables
            foreach ($objArrayForeignFeatures as $strForeignFeature)
            {
                $strAssociatedTable = $this->getRelationTableName(
                    $pstrFeature,
                    $strForeignFeature
                );
                
                $objArraySQL = array();
                array_push($objArraySQL, "SELECT $strAssociatedTable.*");
                array_push($objArraySQL, "FROM $strTable, $strAssociatedTable");
                array_push(
                    $objArraySQL,
                    'WHERE '.$strAssociatedTable.'.'.$pstrFeature.'_id = '
                );
                array_push(
                    $objArraySQL,
                    $strTable.'.id'
                );

                if ($iSingleRecord) 
                {
                    array_push(
                        $objArraySQL,
                        " AND ".$pstrFeature."_id='".$iSingleRecord."'"
                    );
                }
                else if ($objArraySpecificRecords) 
                {
                    $objArrayWhere = array();
                    $iLen = count($objArraySpecificRecords);
                    for ($iIter=0; $iIter<$iLen; $iIter++)
                    {
                        $objArrayWhere[$iIter] = $pstrFeature."_id='".$objArraySpecificRecords[$iIter]."'";
                    }
                    array_push(
                        $objArraySQL,
                        " AND ( ".implode(" OR ", $objArrayWhere)." )"
                    );
                }

                if (in_array($strForeignFeature, $objArrayForeignOrdering)) 
                {
                    array_push(
                        $objArraySQL,
                        'ORDER BY '.$strAssociatedTable.'."order"'
                    );
                }

                $strSQL = implode(" ", $objArraySQL);

                $objArrayAssociatedRecords = array();
                $objResult = pg_query($this->m_objDB, $strSQL);
                if ($objResult) 
                {
                    while (($objArrayRow = pg_fetch_assoc($objResult)) !== false)
                    {
                        array_push($objArrayAssociatedRecords, $objArrayRow);
                    }
                    pg_free_result($objResult);

                    $iNumRec = count($objArrayAssociatedRecords);
                    for ($iIter=0; $iIter<$iNumRec; $iIter++)
                    {
                        if (!isset($objArrayRecords[$objArrayAssociatedRecords[$iIter][$pstrFeature.'_id']][$strForeignFeature])) 
                        {
                            $objArrayRecords[$objArrayAssociatedRecords[$iIter][$pstrFeature.'_id']][$strForeignFeature] = array();
                        }
                        array_push($objArrayRecords[$objArrayAssociatedRecords[$iIter][$pstrFeature.'_id']][$strForeignFeature], $objArrayAssociatedRecords[$iIter][$strForeignFeature.'_id']);
                    }
                }
            }
        }
        else
        {
            $strMessage = "An unknown error occured while querying the DB.";
            throw new Exception($strMessage);
        }

        if ($iSingleRecord) 
        {
            return array_shift($objArrayRecords);
        }
        else
        {
            return $objArrayRecords;
        }
    }

    /**
    * Set a new property to an array.  If the property already exists, create
    *     a 'parent' property with the same name + an 's' at the end and push
    *     the values into it.  If the 'parent' array already exists, simply push
    *     it.
    *
    * @param array  &$pobjArrayOptions Array of options
    * @param string $pstrName          The name of the option to set
    * @param object $pobjValue         The value of the new option to set
    * 
    * @return void
    */
    protected function setOptionValue(
        array &$pobjArrayOptions, $pstrName, $pobjValue
    ) {

        $strParent = $pstrName."s";

        if (isset($pobjArrayOptions[$strParent])) 
        {
            array_push($pobjArrayOptions[$strParent], $pobjValue);
        }
        else
        {
            // if option was already set, create the
            // 'parent' array and unset old option
            if (isset($pobjArrayOptions[$pstrName])) 
            {
                $objArrayOption = array();
                                    
                // push old value and unset it
                array_push($objArrayOption, $pobjArrayOptions[$pstrName]);
                unset($pobjArrayOptions[$pstrName]);

                // push new value
                array_push($objArrayOption, $pobjValue);

                // set new 'parent' array option
                $pobjArrayOptions[$strParent] = $objArrayOption;
            }
            else
            {
                $pobjArrayOptions[$pstrName] = $pobjValue;
            }
        }
    }

    // =========
    //  GETTERS
    // =========

    // -------------------------------
    //  org_geoprisma objects getters
    // -------------------------------

    /**
    * Retourne un object service
    * 
    * @param string $pstrServiceName Service Name
    * 
    * @return org_geoprisma_service_Service 
    */
    public function getService($pstrServiceName)
    { 
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrServiceName);
        
        foreach ($this->m_objArrayService as $objService)
        {
            if ($objService->getName() == $pstrServiceName) 
            {
                return $objService;
            }   
        }

        // we may be asking for a service that doesn't need to be linked to
        // datastores and resources, so try to get it manually.  We need to
        // reconnect to the database first.  If the service is not found at
        // this point, then the 'getRecords' method will throw the according
        // error so no need to do that here.
        $this->connect();
        $objServiceRecord = $this->getRecords(
            $this->getServiceFeatureName(),
            array('singleRecord' => $pstrServiceName)
        );

        $strType = $this->m_objArrayServiceTypes[$objServiceRecord['type']];
        $this->m_objArrayService[$objServiceRecord['id']] = (
            org_geoprisma_service_ServiceFactory::getService(
                $strType, $objServiceRecord
            )
        );
        $this->disconnect();

        return $this->m_objArrayService[$objServiceRecord['id']];
    }
    
    /**
    * Retourne la liste de service disponible service
    * 
    * @return ArrayObject of org_geoprisma_service_Service 
    */
    public function getServices()
    {
        return $this->m_objArrayService;
    } 
    
    /**
    * Retourne un object layout
    * 
    * @param string $pstrLayoutName Layout name
    * 
    * @return org_geoprisma_layout_Layout
    */
    public function getLayout($pstrLayoutName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrLayoutName);
        
        foreach ($this->m_objArrayLayout as $objLayout)
        {
            if ($objLayout->getName() == $pstrLayoutName) 
            {
                return $objLayout; 
            }
        }
        throw new org_geoprisma_exception_LayoutNotFoundException($pstrLayoutName);
    }
    
    /**
    * Retourne la liste des layouts disponible
    * 
    * @return ArrayObject of org_geoprisma_layout_Layout
    */
    public function getLayouts()
    {
        return $this->m_objArrayLayout;
    }   
    
    
    /**
    * Retourne un object datastore
    * 
    * @param string $pstrDatastoreName Datastore name
    * 
    * @return org_geoprisma_datastore_Datastore
    */
    public function getDatastore($pstrDatastoreName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrDatastoreName);
        
        foreach ($this->m_objArrayDatastore as $objDatastore)
        {
            if ($objDatastore->getName() == $pstrDatastoreName) 
            {
                return $objDatastore; 
            }        
        }
        throw new org_geoprisma_exception_DatastoreNotFoundException($pstrDatastoreName);
    }
    
    /**
    * Retourne la liste des datastores disponible
    * 
    * @return ArrayObject of org_geoprisma_datastore_Datastore
    */
    public function getDatastores()
    {
        return $this->m_objArrayDatastore;
    }

    /**
    * Retourne un object widget
    * 
    * @param string $pstrWidgetName Widget name
    * 
    * @return org_geoprisma_widget_Widget
    */
    public function getWidget($pstrWidgetName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrWidgetName); 
        
        foreach ($this->m_objArrayWidget as $objWidget)
        {
            if ($objWidget->getName() == $pstrWidgetName) 
            {
                return $objWidget; 
            }        
        }
        throw new org_geoprisma_exception_WidgetNotFoundException($pstrWidgetName);
    }
    
    /**
    * Retourne la liste des widget disponible
    * 
    * @return ArrayObject of org_geoprisma_widget_Widget
    */
    public function getWidgets()
    {
        return $this->m_objArrayWidget;   
    }

    /**
    * Retorrne un object accessfilter
    * 
    * @param string  $pstrAccessFilterName AccessFilter name
    * @param boolean $pbKey                Returns the key or the value.
    *                                     'value' by default.
    * 
    * @return org_geoprisma_accessfilter_AccessFilter
    */
    public function getAccessFilter($pstrAccessFilterName, $pbKey=false)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrAccessFilterName); 
        
        foreach ($this->m_objArrayAccessFilter as $iKey => $objAccessFilter)
        {
            if ($objAccessFilter->getName() == $pstrAccessFilterName) 
            {
                return ($pbKey) ? $iKey : $objAccessFilter ;
            }        
        }
        throw new org_geoprisma_exception_OptionNotFoundException($pstrAccessFilterName);
    }
    
    /**
    * Retorrne la liste des accessfilter disponible
    * 
    * @return ArrayObject of org_geoprisma_accessfilter_AccessFilter
    */
    public function getAccessFilters()
    {
        return $this->m_objArrayAccessFilter;   
    }
    
    
    /**
    * Retourne un object resource
    * 
    * @param string $pstrResourceName Resource name
    * 
    * @return org_geoprisma_resource_Resource
    */
    public function getResource($pstrResourceName)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrResourceName);
        
        foreach ($this->m_objArrayResource as $objResource)
        {
            if ($objResource->getName() == $pstrResourceName) 
            {
                return $objResource; 
            }        
        }
        throw new org_geoprisma_exception_ResourceNotFoundException($pstrResourceName);        
    }
    
    /**
    * Retourne la liste des resources disponible du mapcontext
    * 
    * @return ArrayObject of org_geoprisma_resource_Resource
    */
    public function getResources()
    {
        return $this->m_objArrayResource;
    }
    
    /**
    * Retourne la liste de tous les resources disponibles.
    * 
    * @return ArrayObject of org_geoprisma_resource_Resource
    */
    public function getOtherResources()
    {
        if (!count($this->m_objArrayOtherResource)) 
        {
            $this->loadOtherResources();
        }
        return $this->m_objArrayOtherResource;
    }

    /**
    * Get a specific field object using its id
    * 
    * @param string $piFieldId Field id
    * 
    * @return org_geoprisma_field_Field
    */
    public function getField($piFieldId)
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($piFieldId);
        
        foreach ($this->m_objArrayField as $objField)
        {
            if ($objField->getId() == $piFieldId) 
            {
                return $objField; 
            }        
        }
        throw new org_geoprisma_exception_FieldNotFoundException($piFieldId);
    }
    
    /**
    * Retourne la liste des fields disponible
    * 
    * @return ArrayObject of org_geoprisma_field_Field
    */
    public function getFields()
    {
        return $this->m_objArrayField;
    }

    /**
    * Get org_geoprisma_widget_MapFishLayerTree widget
    *
    * Only used if isUsingLegacyWidgets() == true
    *
    * @return org_geoprisma_widget_MapFishLayerTree or false if not found
    */
    public function getMapFishLayerTreeWidget() 
    {
        foreach ($this->getWidgets() as $objWidget) 
        {
            if ($objWidget->getType() == org_geoprisma_widget_MapFishLayerTreeWidget::MAP_FISH_LAYER_TREE_WIDGET) 
            {
                return $objWidget;
            }
        }

        return false;
    }

    // ---------------------
    //  utility get methods
    // ---------------------

    /**
    * return instanceof org_geoprisma_config_PGSQLMapContextConfig
    * return instanceof org_geoprisma_config_Config 
    * return instanceof com_borealis_foundation_util_Singleton
    * 
    * @return org_geoprisma_config_PGSQLMapContextConfig 
    */
    public static function getInstance()
    {
        if (self::$s_objInstance === null) 
        {
            self::$s_objInstance = new org_geoprisma_config_PGSQLMapContextConfig();
        }

        return self::$s_objInstance;
    }

    /**
    * Returns an array of foreign ids from a record list
    * 
    * @param array  $pobjArrayRecords   array of records
    * @param string $pstrForeignFeature name of the foreign table
    * 
    * @return array
    */
    protected function getForeignIds($pobjArrayRecords, $pstrForeignFeature)
    {
        $objArrayForeignIds = array();
        foreach ($pobjArrayRecords as $objRecord)
        {
            if (isset($objRecord[$pstrForeignFeature])) 
            {
                foreach ($objRecord[$pstrForeignFeature] as $iForeignId)
                {
                    if (!in_array($iForeignId, $objArrayForeignIds)) 
                    {
                        array_push($objArrayForeignIds, $iForeignId);
                    }
                }
            }
            else if (isset($objRecord[$pstrForeignFeature."_id"])) 
            {
                $iForeignId = $objRecord[$pstrForeignFeature."_id"];
                if (!in_array($iForeignId, $objArrayForeignIds)) 
                {
                    array_push($objArrayForeignIds, $iForeignId);
                }
            }
        }

        asort($objArrayForeignIds);

        return $objArrayForeignIds;
    }

    /**
    * Browse each resource option.  If it's an option ending by any servicetype,
    *     check if it's equal to given servicetype in parameter.  If not, skip
    *     the option.
    *
    * Then, if the option is inside the 'LayersResourcesOptions' array, use
    *     the alternate name (the key is the resource option, the value is the
    *     related layer option to use instead).
    *
    * Finally, push the option inside an array.  In the end, this method returns
    *     the complete list of options from a resource for a specific layer
    *     given a servicetype.
    * 
    * @param array  $pobjArrayResourceOptions Array of options of a specific
    *                                         org_geoprisma_resource_Resource
    *                                         object.
    * @param string &$pstrServiceType         Service type
    * 
    * @return array
    */
    public function getLayerOptionsFromResourceOptions(
        $pobjArrayResourceOptions, &$pstrServiceType
    ) {
        $objArrayLayerOptions = array();
        $objArrayLayerResourceOptions = $this->getLayerResourceOptions();
        $objArrayServiceTypes = $this->getServiceTypes();

        foreach ($pobjArrayResourceOptions as $objResourceOption) 
        {
            $bSkipOption = false;
            $bAllowReplace = false;
            $strLayerOptionName = $objResourceOption->getName();

            // check if the option ends with a 'servicetype' string.  If it
            // does, make sure it's the same as the service type in params, els
            // skip it
            foreach ($objArrayServiceTypes as $strServiceType) 
            {
                $strPattern = "/$strServiceType$/";

                preg_match(
                    $strPattern,
                    $strLayerOptionName,
                    $objArrayMatches
                );

                if (count($objArrayMatches) == 1) 
                {
                    if ($objArrayMatches[0] != $pstrServiceType) 
                    {
                        $bSkipOption = true;
                        break;
                    }
                    else
                    {
                        $strLayerOptionName 
                            = substr(
                                $strLayerOptionName,
                                0,
                                strlen($pstrServiceType) * -1
                            );

                        // since the option is set specifically for this 
                        // service type, we allow replacement of existing
                        // options with the same name.
                        $bAllowReplace = true;
                    }
                }
            }

            if ($this->isUsingLegacyWidgets()) 
            {
                $objArrayLayerResourceOptions
                    = $this->getLayerResourceOptions();

                // if option is set LayerResourceOptions array, then use the
                // alternative name instead
                if (isset($objArrayLayerResourceOptions[$strLayerOptionName])) 
                {
                    $strLayerOptionName 
                        = $objArrayLayerResourceOptions[$strLayerOptionName];
                }
            }

            if ($bSkipOption) 
            {
                continue;
            }

            // if option is set LayerResourceOptions array, then use the
            // alternative name instead
            if (isset($objArrayLayerResourceOptions[$strLayerOptionName])) 
            {
                $strLayerOptionName 
                    = $objArrayLayerResourceOptions[$strLayerOptionName];
            }

            // set new option.  Skip if already set unless 'allow replace' flag
            // is true

            if (!isset($objArrayLayerOptions[$strLayerOptionName]) 
                || $bAllowReplace
            ) 
            {
                $objArrayLayerOptions[$strLayerOptionName] 
                    = $objResourceOption->getValue();
            }
        }

        return $objArrayLayerOptions;
    }

    /**
    * Returns the array of clone names.  If the 'key' parameter is set, returns
    *     only a portion of the array (if it exists, else returns false).
    *
    * @param string $pstrKey Optional key refering to an orignal name.  Defaults
    *                        to false.
    * 
    * @return array or false
    */
    protected function getCloneNames($pstrKey=false) 
    {
        $objArrayReturn = false;

        if ($pstrKey) 
        {
            if (isset($this->m_objArrayClonedWidgetNames[$pstrKey])) 
            {
                $objArrayReturn = $this->m_objArrayClonedWidgetNames[$pstrKey];
            }
        }
        else
        {
            $objArrayReturn = $this->m_objArrayClonedWidgetNames;
        }
        
        return $objArrayReturn;
    }

    /**
    * Returns how many resources the current loaded mapcontext has and which
    *     resources and actions the current logged role has access to (their names).
    *
    * @param array $pobjArrayActions Actions.  In order to be considered
    *                                'authorized', the role must have all 
    *                                action defined there allowed.
    * @param array $pobjArrayOptions Options for this method
    * 
    * @return array containing 2 keys : 'total' (integer), 'authorized'
    *               (array) and 'authorizedActions' (array)
    */
    public function getAuthorizedResources(
        $pobjArrayActions=array("read"), 
        $pobjArrayOptions=array('otherResources' => false)
    ) {
        $objArrayAuthorizedResources = array();
        $objArrayAuthorizedResourcesActions = array();
        $bOtherResources = $pobjArrayOptions['otherResources'];
        if ($bOtherResources) 
        {
            $strGetResourceFunction = 'getOtherResources';
        }
        else
        {
            $strGetResourceFunction = 'getResources';
        }

        foreach ($this->$strGetResourceFunction() as $objResource) 
        {
            $objArrayAuthorizedResourcesActions[$objResource->getName()] = array();
            $bAllAuthorized = true;
            foreach ($pobjArrayActions as $strAction) 
            {
                $bIsAuthorized
                    = org_geoprisma_acl_ACLFactory::getACL()->isAuthorized(
                        $objResource, $strAction
                    );

                if (!$bIsAuthorized) 
                {
                    $bAllAuthorized = false;
                }
                else
                {
                    $objArrayAuthorizedResourcesActions[$objResource->getName()][$strAction] = true;
                }
            }

            if ($bAllAuthorized) 
            {
                array_push(
                    $objArrayAuthorizedResources,
                    $objResource->getName()
                );
            }
        }

        return array(
            'total' => count($objArrayAuthorizedResources),
            'authorized' => $objArrayAuthorizedResources,
            'authorizedActions' => $objArrayAuthorizedResourcesActions
        );
    }

     /**
     * Get reource name strings from current request (from the 'osmresource'
     *     parameter).  This is the first method called by this config.
     *
     *     If there ARE resources in the url, that means the access is made from
     *     the proxy, so we don't need to load everything, just the requested
     *     Resources, their DataStores and Services.
     *
     * @return array
     */
    public function getResourceNamesFromRequest()
    {
        $objArrayResources = array();

        if (isset($_REQUEST) && isset($_REQUEST['osmresource'])) 
        {
            if (is_array($_REQUEST['osmresource'])) 
            {
                $objArrayResources = $_REQUEST['osmresource'];
            }
            else
            {
                $objArrayResources = array($_REQUEST['osmresource']);    
            }
        }

        return $objArrayResources;
    }

    // ------------------------
    //  other property getters
    // ------------------------

    /**
    * Get connection string.
    *
    * @return string
    */
    public static function getConnectionString()
    {
        return self::$s_strConnection;
    }


    /**
    * Returns all service types
    * 
    * @return array
    */
    public function getServiceTypes()
    {
        return $this->m_objArrayServiceTypes;
    }

    /**
    * Get config session id
    *
    * @return integer
    */
    public static function getCurrentSession()
    {
        return self::$s_iCurrentSession;
    }

    /**
    * Get config session name
    *
    * @return string
    */
    public static function getCurrentSessionName()
    {
        return self::$s_strCurrentSessionName;
    }

    /**
    * Get config mapcontext id
    *
    * @return integer
    */
    public static function getCurrentMapContext()
    {
        return self::$s_iCurrentMapContext;
    }

    /**
    * Get config mapcontext name
    *
    * @return string
    */
    public static function getCurrentMapContextName()
    {
        return self::$s_strCurrentMapContextName;
    }

    /**
    * Get config application id
    *
    * @return integer
    */
    public static function getCurrentApplication()
    {
        return self::$s_iCurrentApplication;
    }

    /**
    * Get config application name
    *
    * @return string
    */
    public static function getCurrentApplicationName()
    {
        return self::$s_strCurrentApplicationName;
    }

    /**
    * Get service feature name
    *
    * @return string
    */
    public static function getServiceFeatureName()
    {
        return self::$s_strServiceFeature;
    }

    /**
    * Get datastore feature name
    *
    * @return string
    */
    public static function getDataStoreFeatureName()
    {
        return self::$s_strDataStoreFeature;
    }

    /**
    * Get resource feature name
    *
    * @return string
    */
    public static function getResourceFeatureName()
    {
        return self::$s_strResourceFeature;
    }

    /**
    * Get field feature name
    *
    * @return string
    */
    public static function getFieldFeatureName()
    {
        return self::$s_strFieldFeature;
    }

    /**
    * Get mapcontext feature name
    *
    * @return string
    */
    public static function getMapContextFeatureName()
    {
        return self::$s_strMapContextFeature;
    }

    /**
    * Get widget feature name
    *
    * @return string
    */
    public static function getWidgetFeatureName()
    {
        return self::$s_strWidgetFeature;
    }

    /**
    * Get widget feature name
    *
    * @return string
    */
    public static function getAccessFilterFeatureName()
    {
        return self::$s_strAccessFilterFeature;
    }

    /**
    * Get application feature name
    *
    * @return string
    */
    public static function getApplicationFeatureName()
    {
        return self::$s_strApplicationFeature;
    }

    /**
    * Get session feature name
    *
    * @return string
    */
    public static function getSessionFeatureName()
    {
        return self::$s_strSessionFeature;
    }

    /**
    * Get table prefix string
    *
    * @return string
    */
    public static function getTablePrefix()
    {
        return self::$s_strTablePrefix;
    }

    /**
    * Get option table suffix string
    *
    * @return string
    */
    public static function getOptionTableSuffix()
    {
        return self::$s_strTableOptionSuffix;
    }

    /**
    * Get table name using a given feature name.
    *
    * @param string $pstrFeatureName Name of the feature
    *
    * @return string
    */
    public static function getTableName($pstrFeatureName)
    {
        return self::getTablePrefix().$pstrFeatureName;
    }

    /**
    * Get option table name using a given feature name.
    *
    * @param string $pstrFeatureName Name of the feature
    *
    * @return string
    */
    public static function getOptionTableName($pstrFeatureName)
    {
        return self::getTablePrefix().$pstrFeatureName.self::getOptionTableSuffix();
    }

    /**
    * Get relation table name using a given master feature name and a slave
    *     feature name.
    *
    * @param string $pstrMasterFeatureName Name of the master feature
    * @param string $pstrSlaveFeatureName  Name of the slave feature
    *
    * @return string
    */
    public static function getRelationTableName(
        $pstrMasterFeatureName, $pstrSlaveFeatureName
    ) {
        return self::getTablePrefix().$pstrMasterFeatureName.$pstrSlaveFeatureName;
    }

    /**
    * Get option table name using a given feature name.
    *
    * @param string $pstrMasterFeatureName Name of the master feature
    * @param string $pstrSlaveFeatureName  Name of the slave feature
    *
    * @return string
    */
    public static function getRelationOptionTableName(
        $pstrMasterFeatureName, $pstrSlaveFeatureName
    ) {
        return self::getTablePrefix().$pstrMasterFeatureName.$pstrSlaveFeatureName.self::getOptionTableSuffix();
    }

    /**
    * Get whether the config should use the 'legacy' widgets
    *     (Map and MapFishLayerTree) or not (MapPanel and Layer).
    *
    * @return boolean
    */
    public static function isUsingLegacyWidgets()
    {
        return self::$s_bUseLegacyWidgets;
    }

    /**
    * Returns the layer 'resource options' array.
    *
    * Only used when isUsingLegacyWidgets() == true
    * 
    * @return array
    */
    public function getLayerResourceOptions() 
    {
        return $this->m_objArrayLayerResourceOptions;
    }

    // =========
    //  SETTERS
    // =========

    /**
    * Set connection string.
    *
    * @param string $pstrConnection Connection string used when connecting to
    *                               database
    *
    * @return void
    */
    public static function setConnectionString($pstrConnection)
    {
        self::$s_strConnection = $pstrConnection;
    }

    /**
    * Set config session id
    *
    * @param integer $piValue Value
    *
    * @return void
    */
    public static function setCurrentSession($piValue)
    {
        self::$s_iCurrentSession = $piValue;
    }

    /**
    * Set config session name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setCurrentSessionName($pstrValue)
    {
        self::$s_strCurrentSessionName = $pstrValue;
    }

    /**
    * Set config mapcontext id
    *
    * @param integer $piValue Value
    *
    * @return void
    */
    public static function setCurrentMapContext($piValue)
    {
        self::$s_iCurrentMapContext = $piValue;
    }

    /**
    * Set config mapcontext name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setCurrentMapContextName($pstrValue)
    {
        self::$s_strCurrentMapContextName = $pstrValue;
    }

    /**
    * Set config application id
    *
    * @param integer $piValue Value
    *
    * @return void
    */
    public static function setCurrentApplication($piValue)
    {
        self::$s_iCurrentApplication = $piValue;
    }

    /**
    * Set config application name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setCurrentApplicationName($pstrValue)
    {
        self::$s_strCurrentApplicationName = $pstrValue;
    }

    /**
    * Set service feature name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setServiceFeatureName($pstrValue)
    {
        self::$s_strServiceFeature = $pstrValue;
    }

    /**
    * Set datastore feature name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setDataStoreFeatureName($pstrValue)
    {
        self::$s_strDataStoreFeature = $pstrValue;
    }

    /**
    * Set resource feature name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setResourceFeatureName($pstrValue)
    {
        self::$s_strResourceFeature = $pstrValue;
    }

    /**
    * Set field feature name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setFieldFeatureName($pstrValue)
    {
        self::$s_strFieldFeature = $pstrValue;
    }

    /**
    * Set mapcontext feature name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setMapContextFeatureName($pstrValue)
    {
        self::$s_strMapContextFeature = $pstrValue;
    }

    /**
    * Set widget feature name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setWidgetFeatureName($pstrValue)
    {
        self::$s_strWidgetFeature = $pstrValue;
    }


    /**
    * Set accessfilter feature name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setAccessFilterFeatureName($pstrValue)
    {
        self::$s_strAccessFilterFeature = $pstrValue;
    }

    /**
    * Set application feature name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setApplicationFeatureName($pstrValue)
    {
        self::$s_strApplicationFeature = $pstrValue;
    }

    /**
    * Set session feature name
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setSessionFeatureName($pstrValue)
    {
        self::$s_strSessionFeature = $pstrValue;
    }

    /**
    * Set table prefix string
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setTablePrefix($pstrValue)
    {
        self::$s_strTablePrefix = $pstrValue;
    }

    /**
    * Set option table suffix string
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function setOptionTableSuffix($pstrValue)
    {
        self::$s_strTableOptionSuffix = $pstrValue;
    }
    /**
    * Set legacy widget using (flag)
    *
    * @param string $pstrValue Value
    *
    * @return void
    */
    public static function useLegacyWidgets($pstrValue)
    {
        self::$s_bUseLegacyWidgets = $pstrValue;
    }
}

?>
