<?php
/**
* Récupère les informations de configuration d'un fichier xml avec workspace   
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Récupère les informations de configuration d'un fichier xml avec workspace
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Config  
* @author     Pascal Martin 
*/ 
class org_geoprisma_config_XMLWorkspaceConfig extends org_geoprisma_config_XMLConfig
{
    /**
    * @var org_geoprisma_config_XMLWorkspaceConfig
    */
    private static $s_objInstance = null; 
    
    private static $s_strCurrentWorkspace = '';
  
    private $m_strLayout = null;
    /**
    * Construit une nouvelle instance de org_geoprisma_config_XMLWorkspaceConfig
    * Force l'utilisation de getInstance (Singleton)
    * 
    * @throws org_geoprisma_exception_BadXMLConfigFile
    * 
    * @return org_geoprisma_config_XMLWorkspaceConfig
    */
    private function __construct()
    {    
        $this->load();           
    }
    
    /**
    * return instanceof org_geoprisma_config_XMLWorkspaceConfig
    * return instanceof org_geoprisma_config_Config 
    * return instanceof com_borealis_foundation_util_Singleton
    * 
    * @return org_geoprisma_config_XMLWorkspaceConfig 
    */
    public static function getInstance()
    {
        if (self::$s_objInstance === null) 
        {
            self::$s_objInstance = new org_geoprisma_config_XMLWorkspaceConfig();
        }
        self::$s_objInstance->checkReload();
        return self::$s_objInstance;
    }

    /**
    * Return the layout name
    *
    * @param org_geoprisma_layout_Layout $pstrDefaultLayout Layout
    * 
    * @return string Layout name 
    */
    public function getLayoutName($pstrDefaultLayout)
    {
        if ($this->m_strLayout != null) 
        {
            return $this->m_strLayout;
        }
        else
        {
            return $pstrDefaultLayout;
        }
    }    
  
    /**
    * Load 'org_geoprisma_widget_Widget' object into widget array
    *
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement object
    * 
    * @return void
    */
    protected function loadWidget(SimpleXMLElement &$pobjSimpleXMLElement)
    {
        parent::loadWidget($pobjSimpleXMLElement);
                        
        $objArraySimpleXMLElementWorkspaces = $pobjSimpleXMLElement->xpath("/geoprisma/workspaces/workspace[./name='".self::$s_strCurrentWorkspace."']/map");                 
        if ($objArraySimpleXMLElementWorkspaces !== false && count($objArraySimpleXMLElementWorkspaces) > 0) 
        {
            $strMapName = (string)$objArraySimpleXMLElementWorkspaces[0];
            
            $objArrayWidget = $this->m_objArrayWidget;
            $this->m_objArrayWidget = new ArrayObject();
            
            foreach ($objArrayWidget as $objWidget)
            {
                if ($objWidget instanceof org_geoprisma_widget_MapWidget) 
                {                    
                    if ($objWidget->getName() == $strMapName) 
                    {
                        $objWidget->setName('DefaultMap');
                        $this->m_objArrayWidget->append($objWidget);
                    }
                }
                else
                {
                    $this->m_objArrayWidget->append($objWidget);
                }                
            } 
        }

        // populate exclusive unbound widgets from workspace widgets tag
        $objArraySimpleXMLElementWorkspaceExclusiveWidgets = $pobjSimpleXMLElement->xpath("/geoprisma/workspaces/workspace[./name='".self::$s_strCurrentWorkspace."']/widgets/widget");
        if ($objArraySimpleXMLElementWorkspaceExclusiveWidgets !== false
            && count($objArraySimpleXMLElementWorkspaceExclusiveWidgets) > 0
        ) 
        {
            foreach ($objArraySimpleXMLElementWorkspaceExclusiveWidgets as $iKey => $objArraySimpleXMLElementWorkspaceExclusiveWidget) 
            {
                array_push(
                    $this->m_objArrayExclusiveUnboundWidgetName,
                    (string)$objArraySimpleXMLElementWorkspaceExclusiveWidget
                );
            }
        }

        // mappanel options overrides in workspace
        $objArraySimpleXMLElementWorkspaceMapPanelOptions = $pobjSimpleXMLElement->xpath("/geoprisma/workspaces/workspace[./name='".self::$s_strCurrentWorkspace."']/mappaneloptions/*");
        if ($objArraySimpleXMLElementWorkspaceMapPanelOptions !== false
            && count($objArraySimpleXMLElementWorkspaceMapPanelOptions) > 0
        ) 
        {
            // find first mappanel widget (only one mappanel widget supported)
            $objMapPanelWidget = false;
            foreach ($this->m_objArrayWidget as $objWidget)
            {
                if ($objWidget->getType() == "mappanel") 
                {
                    $objMapPanelWidget = $objWidget;
                    break;
                }
            }
            if ($objMapPanelWidget) 
            {
                foreach ($objArraySimpleXMLElementWorkspaceMapPanelOptions as $iKey => $objArraySimpleXMLElementWorkspaceMapPanelOption) 
                {
                    $objMapPanelWidget->addOrReplaceOption(
                        new org_geoprisma_option_Option(
                            $objArraySimpleXMLElementWorkspaceMapPanelOption->getName(),
                            $this->simpleXMLToArray(
                                $objArraySimpleXMLElementWorkspaceMapPanelOption
                            )
                        )
                    );
                }
            }
        }
    }
  
    /**
    * Charge le tableau d'object org_geoprisma_resource_Resource 
    * a partir du document xml de configuration
    * 
    * @param SimpleXMLElement &$pobjSimpleXMLElement SimpleXMLElement du fichier de config
    * 
    * @return void
    */
    protected function loadResource(SimpleXMLElement &$pobjSimpleXMLElement)
    {
        $this->m_objArrayResource = new ArrayObject();
                    
        $objArraySimpleXMLElementWorkspaces = $pobjSimpleXMLElement->xpath("/geoprisma/workspaces/workspace[./name='".self::$s_strCurrentWorkspace."']"); 
        if ($objArraySimpleXMLElementWorkspaces !== false && count($objArraySimpleXMLElementWorkspaces) > 0) 
        {
            $objSimpleXMLElementWorkspace = $objArraySimpleXMLElementWorkspaces[0];
            if ($objSimpleXMLElementWorkspace->layout != null) 
            {
                $this->m_strLayout = (string)$objSimpleXMLElementWorkspace->layout;
            } 
            foreach ($objSimpleXMLElementWorkspace->resources->resource as $objSimpleXMLElementWorkspaceResource)
            {
                $strResourceName = (string)$objSimpleXMLElementWorkspaceResource->name;
                
                $objArraySimpleXMLElementResources = $pobjSimpleXMLElement->xpath("/geoprisma/resources/resource[./name='$strResourceName']"); 
        
                if (count($objArraySimpleXMLElementResources) > 0) 
                {   
                    $objSimpleXMLElementResource = $objArraySimpleXMLElementResources[0];

                    $strResourceTitle = (string)$objSimpleXMLElementResource->title;
                    
                    // Make ressource
                    $objArrayDatastore = new ArrayObject(); 
                    $objArrayWidget = new ArrayObject();
                    $objArrayAccessFilter = new ArrayObject();
                    $objArrayOption = array();
                    $objArrayResourceOption = new ArrayObject();
                    $objArrayField = new ArrayObject();
                    
                    // Resource datastore from ressources
                    foreach ($objSimpleXMLElementResource->datastores->datastore as $strDatastoreName)
                    {
                        $objArrayDatastore->append($this->getDatastore((string)$strDatastoreName));   
                    }
                    
                    // Resource accessfilter from ressources
                    if($objSimpleXMLElementResource->accessfilters && $objSimpleXMLElementResource->accessfilters->accessfilter)
                    {
                        foreach ($objSimpleXMLElementResource->accessfilters->accessfilter as $strAccessFilterName)
                        {
                            $objArrayAccessFilter->append($this->getAccessFilter((string)$strAccessFilterName));
                        }
                    }
                    
                    // Resource option from ressources
                    foreach ($objSimpleXMLElementResource->xpath('options/*') as $objSimpleXMLElementOption) 
                    {
                        $objArrayOption[$objSimpleXMLElementOption->getName()]
                            = new org_geoprisma_option_Option(
                                $objSimpleXMLElementOption->getName(),
                                $this->simpleXMLToArray(
                                    $objSimpleXMLElementOption
                                )
                            );
                    }
                    
                    // Resource option from workspaces (overrides)
                    foreach ($objSimpleXMLElementWorkspaceResource->xpath('options/*') as $objSimpleXMLElementOption) 
                    {
                        $objArrayOption[$objSimpleXMLElementOption->getName()]
                            = new org_geoprisma_option_Option(
                                $objSimpleXMLElementOption->getName(),
                                $this->simpleXMLToArray(
                                    $objSimpleXMLElementOption
                                )
                            );
                    }

                    // create resource options array object
                    foreach ($objArrayOption as $strName => $objOption) 
                    {
                        $objArrayResourceOption->append($objOption);
                    }

                    // resource fields (no overrides in workspace)
                    if ($objSimpleXMLElementResource->fields) 
                    {
                        foreach ($objSimpleXMLElementResource->fields->field as $objField)
                        {
                            if ((string)$objField->title) 
                            {
                                $strFieldTitle = (string)$objField->title;
                            }
                            else
                            {
                                $strFieldTitle = (string)$objField->name;
                            }

                            $objArrayField->append(
                                new org_geoprisma_field_Field(
                                    (string)$objField->name,
                                    (string)$objField->name,
                                    $this->simpleXMLToArray(
                                        $objField
                                    ),
                                    $strFieldTitle
                                )
                            );
                        }
                    }

                    // Resource widget from workspaces
                    foreach ($objSimpleXMLElementWorkspaceResource->widgets->widget as $objSimpleXMLElementWidget) 
                    {
                        $strWidgetName = '';
                        if ($objSimpleXMLElementWidget->xpath('./name')) 
                        {
                            $objWidget = $this->getWidget((string)$objSimpleXMLElementWidget->name);

                            // Clone widget to overide option
                            $objWidget = clone $objWidget;
                            $objWidget->setName($objWidget->getName() . '_' . $strResourceName);

                            // Load widget option
                            foreach ($objSimpleXMLElementWidget->xpath('options/*') as $objSimpleXMLElementOption) 
                            {
                                $objWidget->addOrReplaceOption(
                                    new org_geoprisma_option_Option(
                                        $objSimpleXMLElementOption->getName(),
                                        $this->simpleXMLToArray(
                                            $objSimpleXMLElementOption
                                        )
                                    )
                                );                          
                            }    
                            $objArrayWidget->append($objWidget);

                            // if the widget doesn't need to be linked to any
                            // resource and wouldn't normally need to be cloned
                            // remove the original one to avoid having
                            // duplicated widgets
                            if ($objWidget->getMandatoryResourceOptionList() === null 
                                && $objWidget->mustClone() === false
                            ) 
                            {
                                $this->removeWidget(
                                    (string)$objSimpleXMLElementWidget->name
                                );
                            }
                        }
                        else
                        { 
                            $objArrayWidget->append($this->getWidget((string)$objSimpleXMLElementWidget));   
                        }
                    }                                    
                    
                    $this->m_objArrayResource->append(new org_geoprisma_resource_Resource($strResourceName, $objArrayDatastore, $objArrayWidget, $objArrayAccessFilter, $objArrayResourceOption, $strResourceTitle, $objArrayField));
                }
            }
        } 
    }
  
  
    /**
    * Set le workspace présentement utilisé
    * 
    * @param string $pstrCurrentWorkspace Nom du workspace courant
    * 
    * @return void
    */
    public static function setCurrentWorkspace($pstrCurrentWorkspace)
    {
        self::$s_strCurrentWorkspace = $pstrCurrentWorkspace;
    }
    
    /**
    * Retourne le workspace présentement utilisé
    *
    * @return string 
    */
    public static function getCurrentWorkspace()
    {
        return self::$s_strCurrentWorkspace;
    }
    
    
  
}


?>
