<?php
/**
* WFS proxy class
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Etienne Dube
* @copyright 2011, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* WFS proxy class
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Etienne Dube 
*/   
abstract class org_geoprisma_proxy_wfs_WFSProxy extends org_geoprisma_proxy_Proxy
{
    
    const REQUEST_TYPE_POSTXML = 1;
    const REQUEST_TYPE_POST = 2;
    const REQUEST_TYPE_GET = 3;
    
    protected $contentType;
    protected $requestType;
    protected $postRequest;
    
    /**
    * Constructor used by subclasses to initialize a WFS Proxy
    * 
    * @param org_geoprisma_service_Service $pobjService Service
    * 
    * @return org_geoprisma_proxy_Proxy
    */
    public function __construct(org_geoprisma_service_Service $pobjService, $pContentType, $pRequestType, $pPostRequest) 
    {
        parent::__construct($pobjService);
        $this->contentType = $pContentType;
        $this->requestType = $pRequestType;
        $this->postRequest = $pPostRequest;
    }
    
    /**
    * Returns list of layers accessed by the current call
    * 
    * @return ArrayObject
    */
    public function getLayers()
    {
        $objArrayLayers = new ArrayObject();
        
        // is this an XML request ?
        if ($this->requestType == self::REQUEST_TYPE_POSTXML) {
            $objDomDoc = new DOMDocument();
            if ( $objDomDoc->loadXML($this->postRequest) )
            {
                $objSimpleXMLElement = simplexml_import_dom($objDomDoc);
                $namespaces = $objSimpleXMLElement->getDocNamespaces();
                $objSimpleXMLElement->registerXPathNamespace('__empty_ns', ''); 
                $objSimpleXMLElement->registerXPathNamespace('wfs', 'http://www.opengis.net/wfs');
                $arrXpathResult = $objSimpleXMLElement->xpath('/wfs:GetFeature/wfs:Query/@typeName');
                if ($arrXpathResult)
                {
                    $typeNames = (string) $arrXpathResult[0]->typeName;
                    $tok = strtok($typeNames, ',');
                    while ($tok !== false) {
                        // feature type will be in the form "ns:feattype" or "ns:feattype=alias"
                        // pattern from OGC spec:  ((\w:)?\w(=\w)?){1,}
                        $matches = array();
                        $numMatches = preg_match('/^(?:\w+:)?(\w+)(?:=\w+)?$/', $tok, $matches);
                        if ($numMatches > 0)
                        {
                            // the real feature type name (i.e. layer from request) will be in match #1
                            $ftName = $matches[1];
                            $objArrayLayers->append($ftName);
                        }
                        $tok = strtok(',');
                    }
                }
            }
        }
        
        // is this a GET request ?
        else if ($this->requestType == self::REQUEST_TYPE_GET) {
            foreach ($_GET as $strKey => $strValue) 
            {
                if (strtoupper($strKey) == "LAYERS") 
                {
                    $objArrayLayer = new ArrayObject(explode(',', $_GET[$strKey]));
                    break;
                }
            }
        }

        return $objArrayLayers;
    }
    
    /**
    * Return data id
    * 
    * @return integer or null
    */
    public static function getID()
    {
        $strPathInfo = self::getPathInfo();
        $objArrayMatches = array();
        if (preg_match(self::PATH_INFO_REG, $strPathInfo, $objArrayMatches) && count($objArrayMatches) >= self::PATH_INFO_ID_POS) 
        {
            $iID = $objArrayMatches[self::PATH_INFO_ID_POS];
            if (is_numeric($iID)) 
            {                
                return (int)$iID;
            }
        }
        return null;
    }

}
  
?>
