<?php
/**
* Class Proxy de type widget
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Class Proxy de type widget
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Proxy 
* @author     Pascal Martin 
*/     
class org_geoprisma_proxy_widget_WidgetProxy extends org_geoprisma_proxy_Proxy
{
    /**
    * @var org_geoprisma_widget_Widget
    */
    private $m_objWidget = null;
    
    private $m_strMethod = null;
    
    /**
    * Construit un instance de org_geoprisma_proxy_widget_WidgetProxy
    * 
    * @param org_geoprisma_service_Service   $pobjService  Service
    * @param org_geoprisma_resource_Resource $pobjResource Resource
    * @param org_geoprisma_widget_Widget     $pobjWidget   Widget
    * @param string                          $pstrProcess  Process
    * 
    * @return org_geoprisma_proxy_widget_WidgetProxy
    */
    public function __construct(
        $pobjService, 
        $pobjResource,
        $pobjWidget = null,
        $pstrProcess = null
    ) {
        if ($pobjService instanceof org_geoprisma_service_Service
            && $pobjResource instanceof org_geoprisma_resource_Resource
        ) 
        {
            parent::__construct($pobjService);

            $objDatastore = $pobjResource->getDatastore($pobjService);
            $strWidgetType = $objDatastore->getOption('widgettype', false);
            $this->m_strMethod = $objDatastore->getOption('method', false);

            foreach ($pobjResource->getWidgets() as $objWidget)
            {
                if ($objWidget->getType() ==  $strWidgetType) 
                {
                    $this->m_objWidget = $objWidget; 
                    break;   
                }  
            }

            if ($this->m_objWidget == null) 
            {
                 throw new Exception('Le widget ne peut pas être utiliser si il n\'est pas dans la ressource');
            }
        }
        else if ($pobjWidget instanceof org_geoprisma_widget_Widget
            && $pstrProcess != null
        ) 
        {
            $this->m_objWidget = $pobjWidget;
            $this->m_strMethod = $pstrProcess;
        }
        else
        {
            throw new Exception('The widget could not be loaded by the proxy');
        }
    }
    
    /**
    * Retourne le nom de l'action réaliser par le proxy (Create - Read - Update - Delete)
    *                                   
    * @return string
    */
    public function getAction()
    {
        return $this->m_objWidget->getAction(); 
    }

    /**
    * Call the widget method
    * 
    * @return void
    */
    public function process()
    {            
        if (method_exists($this->m_objWidget->getType(), $this->m_strMethod)) 
        {           
            call_user_func(array($this->m_objWidget->getType(), $this->m_strMethod), $this->m_objWidget); 
        }
        else
        {      
            throw new Exception('la methode '.$this->m_strMethod.' n\'existe pas dans la classe de widget '.$this->m_objWidget->getType());
        }        
    }
    
    /**
    * Retourne la liste de layer accèder par la query
    * 
    * @return ArrayObject
    */
    public function getLayers()
    {
        return new ArrayObject();         
    }
}
  
?>
