<?php
/**
* Service de type gymo     
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Service de type gymo 
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Service 
* @author     Pascal Martin 
*/   
class org_geoprisma_service_GYMOService extends org_geoprisma_service_Service
{
    const GYMO_SERVICE = 'gymo';
    const TYPE = 'gymo';
    const TYPE_ID = 4;

    const OSM_HOST = "tile.openstreetmap.org";
    
    private $m_strProvider = null;
    private $m_strAPIKey = null;
    
    /**
    * Construit un instance de org_geoprisma_service_GYMOService
    * 
    * @param array $pobjArray Config array
    * 
    * @return org_geoprisma_service_GYMOService
    */
    public function __construct($pobjArray) 
    {
        // name (using 'id' or 'name')
        if (isset($pobjArray['id'])) 
        {
            $strName = (string)$pobjArray['id'];
        }
        else if (isset($pobjArray['name'])) 
        {
            $strName = (string)$pobjArray['name'];
        }

        // source
        if (isset($pobjArray['source'])) 
        {
            $strSource = (string)$pobjArray['source'];
        }

        // provider
        if (isset($pobjArray['provider'])) 
        {
            $this->m_strProvider = (string)$pobjArray['provider'];
        }
        else if (isset($pobjArray['options']['provider'])) 
        {
            $this->m_strProvider = (string)$pobjArray['options']['provider'];
            unset($pobjArray['options']['provider']);
        }

        // apikey
        if (isset($pobjArray['apikey'])) 
        {
            $this->m_strAPIKey = (string)$pobjArray['apikey'];
        }
        else if (isset($pobjArray['options']['apikey'])) 
        {
            $this->m_strAPIKey = (string)$pobjArray['options']['apikey'];
            unset($pobjArray['options']['apikey']);
        }
        
        parent::__construct($strName, $strSource, self::TYPE, $pobjArray);
    }
    
    /**
    * Return the supported provider for the gymo service
    * 
    * @return string 
    */
    public function getProvider()
    {
        return $this->m_strProvider;   
    }
    
    /**
    * Return the apikey
    * 
    * @return string 
    */
    public function getAPIKey()
    {
        return $this->m_strAPIKey;
    }

    /**
    * Return the OpenLayers.Layer default options using this type of service.
    * 
    * @param org_geoprisma_config_Config &$pobjConfig Config object
    *
    * @return array 
    */
    public function getDefaultLayerOptions(&$pobjConfig)
    {
        $objArrayDefaultLayerOptions = array();

        if ($pobjConfig->isUsingLegacyWidgets()) 
        {
            $objArrayDefaultLayerOptions = array(
                'sphericalmercator' => "true",
                'isbaselayer' => "true"
            );
        }
        else
        {
            $objArrayDefaultLayerOptions = array(
                'sphericalMercator' => "true",
                'isBaseLayer' => "true"
            );
        }

        return $objArrayDefaultLayerOptions;
    }

    /**
    *  Ajout le service courant (this) dans le xml     
    * 
    * @param DOMElement  &$pobjDomElementServices Noeud xml ou le service doit être ajouter
    * @param DOMDocument &$pobjDomDocument        Document XMl ou le noeud xml datastore est ajouter, utiliser pour construire les noeuds
    * 
    * @return void
    */
    public function insertIntoXML(DOMElement &$pobjDomElementServices, DOMDocument &$pobjDomDocument)
    {
        $objDomElementService = $pobjDomDocument->createElement('service'); 
        $pobjDomElementServices->appendChild($objDomElementService);        
        $objDomElementType = $pobjDomDocument->createElement('type', self::GYMO_SERVICE);
        $objDomElementName = $pobjDomDocument->createElement('name', $this->getName());
        $objDomElementAPIKey = $pobjDomDocument->createElement('apikey', $this->getAPIKey());
        $objDomElementProvider = $pobjDomDocument->createElement('provider', $this->getProvider());
        $objDomElementSource = $pobjDomDocument->createElement('source');
        $objDomElementSource->appendChild($pobjDomDocument->createCDATASection($this->getSource()));
        $objDomElementService->appendChild($objDomElementType);
        $objDomElementService->appendChild($objDomElementName); 
        $objDomElementService->appendChild($objDomElementAPIKey);
        $objDomElementService->appendChild($objDomElementProvider);
        $objDomElementService->appendChild($objDomElementSource);

        // options node
        $objDomElementService->appendChild(
            $this->getDomElements(
                'options', $this->getOptions(), $pobjDomDocument
            )
        );          
    }
}
  
?>
