<?php
/**
* Service of httpRequest type
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Frédérick Gagnon
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Service of httpRequest type
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Service 
* @author     Frédérick Gagnon
*/   
class org_geoprisma_service_HttpRequestService extends org_geoprisma_service_Service
{
    const HTTP_SERVICE = 'httprequest';
    const TYPE = 'httprequest';
    const TYPE_ID = 10;

    public $m_objArrayCreateHttpRequests = array();
    public $m_objArrayReadHttpRequests = array();
    public $m_objArrayUpdateHttpRequests = array();
    public $m_objArrayDeleteHttpRequests = array();
    /**
    * Creates a new instance of org_geoprisma_service_HttpRequestService
    * 
    * @param array $pobjArray Config array
    * 
    * @return org_geoprisma_service_HttpRequestService
    */
    public function __construct($pobjArray) 
    {
        if (isset($pobjArray['id']))
        {
            $strName = (string)$pobjArray['id'];
        }
        else
        {
            $strName = (string)$pobjArray['name'];
        }

        $strSource = (string)$pobjArray['source'];

        if (isset($pobjArray['options']['createHttpRequests']))
        {
            $this->m_objArrayCreateHttpRequests = explode(',', (string)$pobjArray['options']['createHttpRequests']);
        }

        if (isset($pobjArray['options']['readHttpRequests']))
        {
            $this->m_objArrayReadHttpRequests = explode(',', (string)$pobjArray['options']['readHttpRequests']);
        }

        if (isset($pobjArray['options']['updateHttpRequests']))
        {
            $this->m_objArrayUpdateHttpRequests = explode(',', (string)$pobjArray['options']['updateHttpRequests']);
        }

        if (isset($pobjArray['options']['deleteHttpRequests']))
        {
            $this->m_objArrayDeleteHttpRequests = explode(',', (string)$pobjArray['options']['deleteHttpRequests']);
        }

        parent::__construct($strName, $strSource, self::TYPE, $pobjArray);
    }

    /**
    * Return the OpenLayers.Layer default options using this type of service.
    *
    * @param org_geoprisma_config_Config &$pobjConfig Config object
    *
    * @return array
    */
    public function getDefaultLayerOptions(&$pobjConfig)
    {
        return array();
    }
   
    /**
    * Add current service in XML document.
    *
    * @param DOMElement  &$pobjDomElementServices XML node where this service
    *                                             must be added
    * @param DOMDocument &$pobjDomDocument        DOMDocument XMl used to create
    *                                             new nodes.
    * 
    * @return void
    */
    public function insertIntoXML(DOMElement &$pobjDomElementServices, DOMDocument &$pobjDomDocument)
    {
        $objDomElementService = $pobjDomDocument->createElement('service'); 
        $pobjDomElementServices->appendChild($objDomElementService);        
        $objDomElementType = $pobjDomDocument->createElement('type', self::HTTP_SERVICE);
        $objDomElementName = $pobjDomDocument->createElement('name', $this->getName());
        $objDomElementService->appendChild($objDomElementType);
        $objDomElementService->appendChild($objDomElementName); 

        // options node
        $objDomElementService->appendChild(
            $this->getDomElements(
                'options', $this->getOptions(), $pobjDomDocument
            )
        );
    }
}
  
?>
