<?php
/**
* MapFishPrint service type
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Alexandre Dube 
* @copyright 2010, Centre de Geomatique du Quebec
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* MapFishPrint service type
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Service 
* @author     Alexandre Dube 
*/
class org_geoprisma_service_MapFishPrintService extends org_geoprisma_service_Service
{
    const WMS_SERVICE = 'mapfishprint';
    const TYPE = 'mapfishprint';
    const TYPE_ID = 7;

    private $m_strFormat = null;
    private $m_strSRS = null;
    
    /**
    * Creates a new instance of org_geoprisma_service_MapFishPrintService
    * 
    * @param array $pobjArray Config array
    * 
    * @return org_geoprisma_service_MapFishPrintService
    */
    public function __construct($pobjArray) 
    {
        if (isset($pobjArray['id'])) 
        {
            $strName = (string)$pobjArray['id'];
        }
        else
        {
            $strName = (string)$pobjArray['name'];
        }

        $strSource = (string)$pobjArray['source'];
        
        parent::__construct($strName, $strSource, self::TYPE, $pobjArray);
    }

    /**
    * Return the OpenLayers.Layer default options using this type of service.
    * Since this service won't create any layers, it returns an empty array.
    * 
    * @param org_geoprisma_config_Config &$pobjConfig Config object
    *
    * @return array 
    */
    public function getDefaultLayerOptions(&$pobjConfig)
    {
        return array();
    }
    
    /**
    * Add current service in XML document.
    * 
    * @param DOMElement  &$pobjDomElementServices XML node where this service
    *                                             must be added
    * @param DOMDocument &$pobjDomDocument        DOMDocument XMl used to create
    *                                             new nodes.
    * 
    * @return void
    */
    public function insertIntoXML(
        DOMElement &$pobjDomElementServices,
        DOMDocument &$pobjDomDocument
    ) {
        $objDomElementService = $pobjDomDocument->createElement('service'); 
        $pobjDomElementServices->appendChild($objDomElementService);        
        $objDomElementType = $pobjDomDocument->createElement('type', self::WMS_SERVICE);
        $objDomElementName = $pobjDomDocument->createElement('name', $this->getName());
        $objDomElementService->appendChild($objDomElementType);
        $objDomElementService->appendChild($objDomElementName); 

        // options node
        $objDomElementService->appendChild(
            $this->getDomElements(
                'options', $this->getOptions(), $pobjDomDocument
            )
        );
    }

    /**
    * Get the print capabilities using the source url of the service.  Response
    *     returned is a json string.
    * 
    * @return string
    */
    public function getCapabilities()
    {
        ob_start();
        $objCurl = curl_init($this->getSource());
        curl_setopt($objCurl, CURLOPT_RETURNTRANSFER, 1);
        $strContents = curl_exec($objCurl);
        curl_close($objCurl);
        ob_clean();

        return $strContents;
    }

    /**
    * Get the print capabilities using the source url of the service.  Response
    *     returned is a json string.  Replaces the 'printURL' and 'createURL'
    *     parameters to point to the proxy url instead.
    * 
    * @return string
    */
    public function getCapabilitiesWithProxyURL()
    {
        $objArrayCapabilities = json_decode($this->getCapabilities(), true);

        $objArrayCapabilities['printURL'] = $this->urlAppend(
            org_geoprisma_SettingImpl::getProxyURL(),
            array("osmservice" => $this->getName())
        );

        $objArrayCapabilities['createURL'] = $this->urlAppend(
            org_geoprisma_SettingImpl::getProxyURL(),
            array("osmservice" => $this->getName())
        );

        return json_encode($objArrayCapabilities);
    }
}
  
?>
