<?php
/**
* Constructs the appropriate service object according to the node type in the XML configuration
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Pascal Martin 
* @copyright 2009, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* Constructs the appropriate service object according to the node type in the XML configuration
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Service 
* @author     Pascal Martin 
*/ 
class org_geoprisma_service_ServiceFactory
{

    /**
    * Returns a Service object.
    * 
    * @param string $pstrType  Service type
    * @param array  $pobjArray Config array of the service
    * 
    * @return org_geoprisma_service_Service object
    */
    static public function getService($pstrType, $pobjArray) 
    {
        com_borealis_foundation_util_Assert::assertIsNotEmptyString($pstrType);
        
        switch($pstrType)
        {
            case org_geoprisma_service_WMSService::WMS_SERVICE:
                return new org_geoprisma_service_WMSService($pobjArray);
                break;
            case org_geoprisma_service_FeatureServerService::FEATURESERVER_SERVICE:
                return new org_geoprisma_service_FeatureServerService($pobjArray);                      
                break;    
            case org_geoprisma_service_MapServerService::MAPSERVER_SERVICE:
                return new org_geoprisma_service_MapServerService($pobjArray);                      
                break;
            case org_geoprisma_service_TileCacheService::TILECACHE_SERVICE:
                return new org_geoprisma_service_TileCacheService($pobjArray);                      
                break;
            case org_geoprisma_service_GYMOService::GYMO_SERVICE:
                return new org_geoprisma_service_GYMOService($pobjArray);
                break;
            case org_geoprisma_service_WidgetService::WIDGET_SERVICE:
                return new org_geoprisma_service_WidgetService($pobjArray);
                break;
            case org_geoprisma_service_MapFishPrintService::TYPE:
                return new org_geoprisma_service_MapFishPrintService($pobjArray);
                break;
            case org_geoprisma_service_FileService::TYPE:
                return new org_geoprisma_service_FileService($pobjArray);
                break;
            case org_geoprisma_service_WFSService::TYPE:
                return new org_geoprisma_service_WFSService($pobjArray);
                break;
            case org_geoprisma_service_WPSService::TYPE:
                return new org_geoprisma_service_WPSService($pobjArray);
                break;
            case org_geoprisma_service_HttpRequestService::TYPE:
                return new org_geoprisma_service_HttpRequestService($pobjArray);
                break;
            case org_geoprisma_service_TMSService::TYPE:
                return new org_geoprisma_service_TMSService($pobjArray);
                break;
        }
        throw new Exception("Unsupported service type: $pstrType");
    }

}

?>
