<?php
/**
* WFS service
*
* PHP versions 5
* 
* @category  PHP
* @package   GeoPrisma  
* @author    Etienne Dube
* @copyright 2011, Boreal - Information Strategies
* @license   http://www.geoprisma.org/license BSD License
* @link      http://www.geoprisma.org
*/

/**
* WFS service
* 
* @category   PHP 
* @package    GeoPrisma
* @subpackage Service 
* @author     Etienne Dube
*/  
class org_geoprisma_service_WFSService extends org_geoprisma_service_Service
{
    const WFS_SERVICE = 'wfs';
    const TYPE = 'wfs';
    const TYPE_ID = 9;

    /**
    * Construit un instance de org_geoprisma_service_FeatureServerService
    * 
    * @param array $pobjArray Config
    * 
    * @return org_geoprisma_service_FeatureServerService
    */
    public function __construct($pobjArray) 
    {
        if (isset($pobjArray['id'])) 
        {
            $strName = (string)$pobjArray['id'];
        }
        else
        {
            $strName = (string)$pobjArray['name'];
        }

        $strSource = (string)$pobjArray['source'];
    
        parent::__construct($strName, $strSource, self::TYPE, $pobjArray);
    }

    /**
    * Return the OpenLayers.Layer default options using this type of service.
    * 
    * @param org_geoprisma_config_Config &$pobjConfig Config object
    *
    * @return array 
    */
    public function getDefaultLayerOptions(&$pobjConfig)
    {
        return array(
            'visibility' => 'false',
            'extractAttributes' => 'true'
        );
    }
    
    /**
    * Add this service instance in the XML file
    * 
    * @param DOMElement  &$pobjDomElementServices XML node where the service is to be added
    * @param DOMDocument &$pobjDomDocument        XML Document where the datastore XML node is added, used to construct the nodes
    * 
    * @return void
    */
    public function insertIntoXML(DOMElement &$pobjDomElementServices, DOMDocument &$pobjDomDocument)
    {
        $objDomElementService = $pobjDomDocument->createElement('service'); 
        $pobjDomElementServices->appendChild($objDomElementService);        
        $objDomElementType = $pobjDomDocument->createElement('type', self::WFS_SERVICE);
        $objDomElementName = $pobjDomDocument->createElement('name', $this->getName());
        $objDomElementService->appendChild($objDomElementType);
        $objDomElementService->appendChild($objDomElementName);

        // options node
        $objDomElementService->appendChild(
            $this->getDomElements(
                'options', $this->getOptions(), $pobjDomDocument
            )
        );
    }

    /**
    * Build and execute a WFS DescribeFeatureType request given a specific
    * typename.
    * 
    * @param string $pstrTypeName The typename to use in the request.
    * 
    * @return string
    */
    public function getDescribeFeatureType($pstrTypeName)
    {
        $objArrayParams = array(
            "SERVICE" => "WFS",
            "REQUEST" => "DescribeFeatureType",
            "VERSION" => "1.0.0",
            "TYPENAME" => $pstrTypeName
        );

        $objCurl = curl_init(
            $this->urlAppend($this->getSource(), $objArrayParams)
        );
        curl_setopt($objCurl, CURLOPT_RETURNTRANSFER, 1);
        $strContents = curl_exec($objCurl);
        curl_close($objCurl);
        ob_clean();

        return $strContents;
    }
}
  
?>
